#ifndef	AM_9519_H
#define	AM_9519_H
/*
@@@  am_9519.h - to use as "..." by drivers of stuff using this chip
@@@  contains all commands of am_9519 chip designed for interrupt 
@@@  control of up to 8 interrupt sources.
@@@  Used in CAMAC branch controller CBD 8210 of CES.
@@@  The chip has two 8 bit ports
@@@	CTRL port	read status register, write command
@@@	DATA port	read register preselected by command
@@@			write command concerning preselected register
@@@			write vector to response memory
*//*
	Status	register	
		S7   Group interrupt	
			0 At least one unmasked IRR bit set
			1           No unmasked IRR bit set
		S6   Enable input
			0 Chip disabled
			1 Chip enabled
		S5   Priority (M0)
			0 Fixed
			1 Rotating
		S4   Interrupt mode (M2)
			0 Interrupt
			1 Polled
		S3   Master mask ( M7 )
			0 Chip disarmed
			1 Chip armed
		S2-0 Binary vector (valid when S7 = 0)
*//*
	Mode 	register
		M7   Master mask
			0 Chip disarmed
			1 Chip armed
		M6-5  Registers access via DATA port
			0 0 ISR
			0 1 IMR
			1 0 IRR
			1 1 ACR
		M4    IREQ polarity
			0 Active low
			1 Active high
		M3    GINT polarity
			0 Active low
			1 Active high
		M2    Vector selection
			0 per line
			1 common vector
		M0    Priority
			0 fixed
			1 round-robin
*//*
	ISR	(interrupt service register	)
		set bit    interrupt ACK'ed
		clr bit    by host or by setting bit in ACR 
	IMR	(interrupt mask register	)
		set bit    interrupt masked  
		clr bit	   interrupt checked
	IRR	(interrupt request register	)
		set bit	   IREQ was busy 
		clr bit	   by ACK
	ACR	(auto clear register		)
		set bit    bit of ISR cleared locally during ACK
		clr bit    bit of ISR has to be cleared by host
*/
/* SET OF COMMANDS via CTRL port	*/
#define	RESET_AMD	0x0
/* joint IRR/IMR commands */
#define	CLR_IRR_IMR	0x10
#define	BCLR_IRR_IMR	0x18	/* to be or'ed with bit val 0-7 */
/* IMR */
#define	CLR_IMR		0x20
#define	BCLR_IMR	0x28 /* to be or'ed with bit val 0-7 */
#define	SET_IMR		0x30
#define	BSET_IMR	0x38 /* to be or'ed with bit val 0-7 */
/* IRR */
#define	CLR_IRR		0x40
#define	BCLR_IRR	0x48 /* to be or'ed with bit val 0-7 */
#define	SET_IRR		0x50
#define	BSET_IRR	0x58 /* to be or'ed with bit val 0-7 */
/* ISR */
#define	HCLR_ISR	0x60 /* clear high bit only 	*/
#define	CLR_ISR		0x68
#define	BCLR_ISR	0x78 /* to be or'ed with bit val 0-7 */
/* MODE */
#define	MOD_LO		0x80 /* to be or'ed with M0-M4 */
#define	MOD_HI		0xa0 /* to be or'ed with OR_xxx and M7_xxx: */
#	define	ON_ISR		0x0
#	define	ON_IMR		0x4
#	define	ON_IRR		0x8
#	define	ON_ACR		0xc
#	define	M7_TST			0x0
#	define	M7_SET			0x1
#	define	M7_CLR			0x10
/*
	ACC_REG_TST	open access to a register 
				chip preserves it's state
	ACC_REG_SET	open access to a register
				chip services interrupts
	ACC_REG_CLR	open access to a register
				chip does not make GINTS to the host

			TST	SET	CLR
*/
#define	ACC_ISR_TST	0xa0
#define	ACC_ISR_SET		0xa1
#define	ACC_ISR_CLR			0xa2

#define	ACC_IMR_TST	0xa4
#define	ACC_IMR_SET		0xa5
#define	ACC_IMR_CLR			0xa6

#define	ACC_IRR_TST	0xa8
#define	ACC_IRR_SET		0xa9
#define	ACC_IRR_CLR			0xaa

#define	ACC_ACR_TST	0xac
#define	ACC_ACR_SET		0xad
#define	ACC_ACR_CLR			0xae

/* PRESELECT FOR WRITING */
#define	WR_IMR		0xb0
#define	WR_ACR		0xc0
#define WR_MEM(cnt,lev)	((0xe0 | ((((cnt) - 1) & 0x3) << 3) | ((lev) & 0x7)))

#endif	AM_9519_H
