#ifndef	__BIT3_DEF_H
#define	__BIT3_DEF_H
/*
    vmehb - VME bus driver of PCI/VME bus adaptor Model 617 of Bit3.
    ----------------------------------------------------------------
    The software structure of the driver is defined here
    Batch of constants & macro's 
    Copyright (C) May 1997  Natalia Kruszynska <natalia@nikhef.nl>
	Thanks for contribution to:
		Andries Brouwer <aeb@cwi.nl>
		Marcin Dalecki  <dalecki@sub944.sub.uni-goettingen.de>
		David Grothe	<dave@gcom.com>
		Henk Peek	<henkp@nikhef.nl>
		Ruud van Wijk   <ruud@nikhef.nl>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/****************************************************************
** PCI configuration of bit3 & AMCC devices
*****************************************************************/
#define PCI_VENDOR_ID_BIT3	0x108a
#define PCI_DEVICE_ID_BIT3_VME	0x1
#define PCI_DEVICE_ID_AMCC_KIT	0x4750


#include	"pci_info.h"	/**/

/****************************************************************
** BIT3 configuration constants	
*****************************************************************/
#define	BIT3_MAJOR		62	
#define	BIT3_IRQ		5
#define BIT3_DMA_AHEAD		0	/* write to/from user	*/
					/* concurrent to DMA	*/
#define VME_READ		0
#define VME_WRITE		2

/****************************************************************
** What is needed to service the request
*****************************************************************/
struct	io_ahead {
/* general control */
	int	pid;		/* caller pid for control	*/
	int	devp;		/* address to devp for multidev	*/
	int	acc;		/* 0 for PIO 1 for DMA		*/
	int	rw;		/* 0 read    2 write		*/
	int	vadr;		/* VME addr			*/
	int	align;		/* align the addr for VME access*/
	volatile	int	error;		/* ev errors on transfer	*/
/* mapping details */
	int	n_map;		/* starting map register	*/
	int	n_len;		/* len of mapping		*/
/* PIO only part */
	int	vam;		/* PIO vme address modifier	*/
	int	off;		/* offset to first page addr	*/
/* DMA part */
	int	addr;		/* PCI addr			*/
	int	dam;		/* DMA vme address modifier	*/
};
/* we will communicate via pointer uio_p (missing unix?)	*/
extern	struct io_ahead *uio_p;
#define	uio_pid		((int)((struct io_ahead *)uio_p)->pid)
#define	uio_devp	((int)((struct io_ahead *)uio_p)->devp)
#define	uio_acc		((int)((struct io_ahead *)uio_p)->acc)
#define	uio_rw 		((int)((struct io_ahead *)uio_p)->rw )
#define	uio_n_map	((int)((struct io_ahead *)uio_p)->n_map)
#define	uio_n_len	((int)((struct io_ahead *)uio_p)->n_len)
#define	uio_n_ctl	((int)((struct io_ahead *)uio_p)->n_ctl)
#define	uio_vadr	((int)((struct io_ahead *)uio_p)->vadr)
#define	uio_align	((int)((struct io_ahead *)uio_p)->align)
#define	uio_error	((int)((struct io_ahead *)uio_p)->error)

#define	uio_vam		((int)((struct io_ahead *)uio_p)->vam)
#define	uio_off		((int)((struct io_ahead *)uio_p)->off)

#define	uio_addr	((int)((struct io_ahead *)uio_p)->addr)
#define	uio_dam		((int)((struct io_ahead *)uio_p)->dam)

/****************************************************************
** What is needed for mmap fragments control & unmap at close fd
*****************************************************************/
struct	vme_frag	{
	int			pid;
	unsigned int		vadr;
	int			from;
	int			pages;
	struct	vme_frag	*next;
};

/****************************************************************
** Global info of bit3 device driver
*****************************************************************/
struct bit3_dev{
	volatile	struct wait_queue	
					*wait_q;
			struct	semaphore	
					sem;
	volatile	int		wakeups;	/* processes to wake	*/
	volatile	int		pio_delay;  
	volatile	int		error;		/* ev errors on transfer	*/
/* config addresses vremapped (ioremapped if necessary) */
	int				base0;
	int				base2;
	int				base3;
/* flags */
	volatile	int		VME_on;
	volatile	int		dma_busy;
	volatile	int		dma_pid;
	volatile	int		irq_used;
	volatile	int		dma_done; 
	volatile	int		slow_control; 
	volatile	int		mappings;
/* stores , now we store the current in aheads as well, no AHEAD yet	*/
	volatile	int		free_ahead;
	volatile	struct  io_ahead	
					aheads[1 + BIT3_DMA_AHEAD];
	struct		vme_frag	*first;
	struct		vme_frag	*next;
};
extern struct bit3_dev *bit3_dev_p;

#define	bit3_wait_struct	((struct wait_queue *)(bit3_dev_p->wait_q))
#define	bit3_wait_q		((struct wait_queue **)&(bit3_dev_p->wait_q))
#define bit3_sem		((struct semaphore)(bit3_dev_p->sem))
#define	bit3_VME_on		(bit3_dev_p->VME_on)
#define	bit3_error		((u32)(bit3_dev_p->error))
#define	bit3_base0		(bit3_dev_p->base0)
#define	bit3_base2		(bit3_dev_p->base2)
#define	bit3_base3		(bit3_dev_p->base3)
#define	bit3_wakeups		(bit3_dev_p->wakeups)
#define	bit3_dma_busy		(bit3_dev_p->dma_busy)
#define	bit3_dma_pid		(bit3_dev_p->dma_pid)
#define	bit3_irq_used		(bit3_dev_p->irq_used)
#define	bit3_slow_control	(bit3_dev_p->slow_control)
#define	bit3_mappings		(bit3_dev_p->mappings)
#define	bit3_pio_delay		(bit3_dev_p->pio_delay)
#define	bit3_dma_done		(bit3_dev_p->dma_done)
#define	bit3_free_ahead		(bit3_dev_p->free_ahead)
#define bit3_ahead(i)		(&bit3_dev_p->aheads[i])
#define bit3_first		((struct vme_frag *)(bit3_dev_p->first))
#define bit3_next		((struct vme_frag *)(bit3_dev_p->next))

/****************************************************************
** Configuration info of vme minors, also for ioctl
*****************************************************************/
struct vme_dev {
/* VME devices configuration data */
	char 	name[10];
	int	minor;
	int	am;	/* VME address modifier			*/
	int	mod;	/* 2 or 4				*/
	int	dmamod;	/* BLOCK PAUSE or NON_BLOCK 		*/
	int	swap;   /* MR_BYTE_SWAP|MR_WORD_SWAP|MR_WBYTE_S.*/
	int	fu;	/* 1 - VME 2 - Dual Port 3 - VME RAM	*/
	int	pio;	/* MAX len serviced by PIO		*/
	u32	maxad;	/* MAX addressing,eg ffff for a16	*/
};
extern	struct vme_dev * vme_dev_p;

#define vme_name     (vme_dev_p->name)
#define vme_mod      (int)(vme_dev_p->mod)
#define vme_am       (int)(vme_dev_p->am)
#define vme_pio      (int)(vme_dev_p->pio)
#define vme_dmamod   (int)(vme_dev_p->dmamod)
#define vme_swap     (int)(vme_dev_p->swap)
#define vme_fu       (int)(vme_dev_p->fu)
#define vme_pio      (int)(vme_dev_p->pio)
#define vme_maxad    (u32)(vme_dev_p->maxad)

/*	__BIT3_DEF_H*/
#endif
