#ifndef CM_H
# 	define	CM_H
#       include "camdef.h"
#       include "camioctl.h"

/*
**  cam.h - to use only in CAMAC driver
**  hardware: sun4c/pti sun4e (sun4m)
*/

#define	reg_t	

/* CES controller registers  structure				*/
typedef	struct	{
	ushort	qq0;
	ushort	ccsr;
	ushort	amd_data[6];
	ushort	qq1;
	ushort	ifr;
	ushort	amd_ctrl[6];
	ushort	qq2;
	ushort	car;
	ushort	qq3;
	ushort	btb_bz;
	ulong	gl;
}cm_t;

typedef	struct	{
	int	so_status;	/* status of slot		*/
	char   *so_ptr;         /* pointer in kern   buffer     */
	int	so_flags;
}cm_s_t;

typedef	struct {
	u_int   pa_kmx;         /* 16- pointer set              */
	caddr_t pa_ptr;
	int     pa_cnt;
}cm_p_t;

typedef	struct	{
	int	si_cminor;	/* device installing interrupt	*/
	int	si_pid;		/* pid of installing process	*/
	struct proc *si_procp;  /* proc ptr for psignal         */
}cm_i_t;

/* CIORES, CIOSHUT 						*/
typedef struct	{
	int	sr_status;	/* status of a crate		*/
	int	sr_crate;	/* crate			*/
	int	sr_func;	/* 0 restore 1 shut 		*/
}cm_r_t;

/****** hardware dependency *************************************/
/* physical base from which CES geographically addresses CAMAC 	*/
#ifdef	linux
#define	CM_VME_BASE	0x800000
#define VME_PAGESIZE	0x1000
#endif
#ifndef linux
#ifdef	sun4e
#define	CM_VME_BASE	0xFF800000
#endif
#ifdef	sun4c
#define	CM_VME_BASE	SB_WINDOW(CAMAC_DEVICE)
#endif	
#ifdef	sun4m
#ifdef  PTVME_DRIVER
#define CM_VME_BASE     0x800000
#else
#define	CM_VME_BASE	SB_WINDOW(CAMAC_DEVICE)
#endif
#endif
#endif
/****************************************************************/

struct  byte            { char abyte; };
struct  usword          { unsigned short  ushort; };
struct  uslong          { unsigned long   ulong; };

/* 
**	CAMAC hard addressing of cm_t control registers:
**      put Cm_regs  at:  CNAF (0,29,0,0)
*/	
#define CCSR            &(Cm_regs->ccsr)
#define CCSR_DEF        0x082c
#define CCSR_ERR        0x2bad

#define SLOT_Z          28      /* slot in all crates   */
#define SLOT_CES        29
#define SLOT_CTRL       30      /* slot in all crates   */

/* 
** VME CBD 8210  CAMAC status macro's.
*/
#define	Q		0x8000	/* Q-response received		*/
#define	X		0x4000	/* X-response received		*/
#define	CAM_TO		0x2000	/* time-out signalled		*/
#define	CAM_BD		0x1000	/* BD read transparent status	*/
#define	MAXI		0x0800	/* mask NO_X interrupt bit	*/
#define	MINTO		0x0020	/* mask internal time-out bit	*/
#define	MLAM		0x0010	/* mask LAM interrupts		*/
#define	MINT4		0x0004	/* mask external interrupt lev 4 */
#define	MINT2		0x0002	/* mask external interrupt lev 2 */

#define	Q_RESPONSE(status)	(  (status) & Q)
#define	NO_Q_RESPONSE(status)	(!((status) & Q))
#define	X_RESPONSE(status)	(  (status) & X)
#define	NO_X_RESPONSE(status)	(!((status) & X))

#define SWDES                   2
/* physical address for a given CAMAC slot */
#ifdef	linux
#define HARD_ADDR(N)            (((N) >> 2) * VME_PAGESIZE * PAGES_PER_4_SLOTS + CM_VME_BASE)
#endif
#ifndef linux
#define HARD_ADDR(N)            (((N) >> 2) * MMU_PAGESIZE * PAGES_PER_4_SLOTS + CM_VME_BASE)
#endif
#define NAFA(N,A,F)             (cm_s_regs->so_ptr + ((A) << 7) + ((F) << 2))
#define	UNAF(N,A,F)		((struct usword *)(NAFA(N,A,F) + SWDES))

#define NO_PANTS(D)             (((D) >> 16) & 0xFFFF)
#define TOPLESS(D)              (((D)      ) & 0xFFFF)

/*
** MAIN_LEVEL is level on which level 3 VME is maintained by the system bus
** it is defined in vme_conf.h and used to preset VME on SPARC's
** Should be equal to SBS_INTR_LEVEL on Sparc 4c/4m arch.
*/
/****** hardware dependencies ****************************************/
#ifdef	sun4e
#define         MAIN_LEVEL      5
#endif
#ifdef	sun4c
#define         MAIN_LEVEL      SBS_INTR_LEVEL
#endif	
#ifdef	sun4m
#define         MAIN_LEVEL      SBS_INTR_LEVEL
#endif	
/*********************************************************************/

/*
** Device oriented macro's
*/
#define  SLOT(node)              MINOR(node->i_rdev)
#define  N_EXT(n,c)             ((n) + ((c) << 5))
#define  N_CR(N)                ((N) & 0x1f)
#define  CRATE(cminor)          (((cminor) >> 5) & 0x3)

/* Interrupts */
#define	_amd_intr_lev(N)	(((N_CR(N)) - 1) % 8)
#define	_amd_intr_cmd(N)	WR_MEM(CAMVECLEN,_amd_intr_lev(N))
#define	_amd_pos(N)		(((N_CR(N)) - 1)/8)

#define	SLOT_CLOSED		0
#define	SLOT_OPENED		2

#endif	CM_H
#ifndef	__BIT3_DEF_H
#define	__BIT3_DEF_H

/****************************************************************
** Global info of bit3 device driver
*****************************************************************/
struct bit3_dev{
	volatile	struct wait_queue	
					*wait_q;
			struct	semaphore	
					sem;
	volatile	int		wakeups;	/* processes to wake	*/
	volatile	int		pio_delay;  
	volatile	int		error;		/* ev errors on transfer	*/
/* config addresses vremapped (ioremapped if necessary) */
	int				base0;
	int				base2;
	int				base3;
/* flags */
	volatile	int		VME_on;
	volatile	int		dma_busy;
	volatile	int		dma_pid;
	volatile	int		irq_used;
	volatile	int		dma_done; 
	volatile	int		slow_control; 
	volatile	int		mappings;
};
extern struct bit3_dev *bit3_dev_p;

#define	bit3_base0		(bit3_dev_p->base0)
#define	bit3_base2		(bit3_dev_p->base2)
#define	bit3_base3		(bit3_dev_p->base3)

/****************************************************************
** Configuration info of vme minors, also for ioctl
*****************************************************************/
struct vme_dev {
/* VME devices configuration data */
	char 	name[10];
	int	minor;
	int	am;	/* VME address modifier			*/
	int	mod;	/* 2 or 4				*/
	int	dmamod;	/* BLOCK PAUSE or NON_BLOCK 		*/
	int	swap;   /* MR_BYTE_SWAP|MR_WORD_SWAP|MR_WBYTE_S.*/
	int	fu;	/* 1 - VME 2 - Dual Port 3 - VME RAM	*/
	int	pio;	/* MAX len serviced by PIO		*/
	u32	maxad;	/* MAX addressing,eg ffff for a16	*/
};
extern	struct vme_dev * vme_dev_p;

#define vme_name     (vme_dev_p->name)
#define vme_mod      (int)(vme_dev_p->mod)
#define vme_am       (int)(vme_dev_p->am)
#define vme_pio      (int)(vme_dev_p->pio)
#define vme_dmamod   (int)(vme_dev_p->dmamod)
#define vme_swap     (int)(vme_dev_p->swap)
#define vme_fu       (int)(vme_dev_p->fu)
#define vme_pio      (int)(vme_dev_p->pio)
#define vme_maxad    (u32)(vme_dev_p->maxad)

/*	__BIT3_DEF_H*/
#endif
