/**      CLASSIC Mind SB_BASE in include/vme_conf.h!
**
@@@	Dec 1997 - starting from:
@@@     CAMAC (Pseudo) driver for SPARC and SPARC engine
@@@	. all interrupts at 0xCA  VME vector (jumpered  on CES)
@@@	  GL register not addressable when no CAMAC crate is on line
@@@     . CAMAC access not covered against power-down on already open device
@@@     . Full branch can be coupled (MAX_CR_CNTR)
@@@     . No protections, multi-access to a given slot (module)
@@@     . 24 single-user interrupts for the whole branch restartable per crate
@@@     . One CES per vme crate, branch number always 0
*/
/* here we have all */
#include        "camces_conf.h"

/*  
@@@	This section should land in camces_conf.h
*/

#define	CAM_MAJOR	63
#define VME_MINOR	2
#define	DELAY(a)	udelay(a)
#define LET_CAMAC_REACT	3
#define	peek(a)		(a)
#define poke(a,v)	(*(a) = v)

extern unsigned int vme_dev_mmap(int,unsigned int,unsigned int);
extern unsigned int vme_dev_unmap(int,unsigned int,unsigned int);

static	int	cam_major;
static	int	cam_irq		= 5;

static  char    Cam_name[]      = {"cm"};
static  cm_t   *Cm_regs         = 0;    /* registers of CES     */
static  int     Cm_attached     = 0;    /* for unloading        */
static  cm_s_t  Cm_s_db[S_NCM];         /* slot  adm            */
static  cm_p_t  Cm_p_db[CM_PAGES];      /* page  adm            */
static  cm_i_t  Cm_i_db[24];            /* lam   adm            */
static  int     Cm_r_db[8];             /* crate adm            */

/* A>15 means 16 bits access    */
#define cm_action(f,l)  ((l) ? \
        (((f) < 16 || (f) > 23) ? (CAM_16|CAM_RD) : (CAM_16|CAM_WR)) : \
        (((f) < 8)  ? CAM_RD :    \
         ((f) < 16) ? CAM_EX :    \
        (((f) < 24) ? CAM_WR :    \
                      CAM_EX)))

#define cm_s_alloc(slot)        (cm_s_t *)&(Cm_s_db[(slot)])
#define cm_p_alloc(slot)        (cm_p_t *)&(Cm_p_db[(slot) >> 2])
#define cm_i_alloc(lam)         (cm_i_t *)&(Cm_i_db[(lam - 1)])

#define mess_log                printk  /* local   terminal             */
#define MAXTRY                  3       /* repeat some strokes          */

#define START_CRATE             -1      /* no protection on cm_open     */
#define START_CRITICAL(addr)    cli()
#define   END_CRITICAL(addr)    sti()

/* PROTOTYPES */

static int  cm_open(struct inode *node,struct file *file);
static int  cm_sl_open(int cminor, int flag);
static void cm_release(struct inode *node,struct file *file) ;
static void cm_close(int cminor);
static int  cm_ioctl(struct inode *node,struct file *file,
			unsigned int cmd, unsigned long arg);
static void cm_poll(int cam_irq,void *dev_id,struct pt_regs *regs);

static int _init_cam();
int camac( int cminor, caddr_t addr);
int lam_allow(ushort cminor);
int lam_ignore(ushort cminor);
int lam_init(ushort cminor, caddr_t addr,int  flag);
void _am_vec_init();

int cm_restart(int cminor,caddr_t addr);
int cm_shut(int cminor,caddr_t addr);

int start_crate(unsigned int crate);
int shut_crate(unsigned int crate);
int cm_getcrate(int cminor, caddr_t addr);
 
int cm_vme_alloc(int cminor);
int cm_vme_free(int cminor);

#define PEEK(adr)       ((struct usword *)(adr))->ushort                /*  */
#define POKE(adr, dat)  (((struct usword *)(adr))->ushort = (dat), 0)   /*  */
inline void ncamread(ushort *tNAF,ushort *tD)
{
	int val;

	val = PEEK(tNAF);
	*(tD)  = TOPLESS(val);
}
inline void  ncamwrite(ushort *tNAF,ushort Dval)
{
	if (POKE(tNAF, Dval) < 0)
	    BASE_TRACE(("CM: BUS ERROR "));
}
inline void ncam24read(ushort *tNAF,ushort *tD)
{
	ncamread( tNAF++,(tD  + 1));
	ncamread( tNAF   , tD);
}
inline void ncam24write(ushort *tNAF,ulong Dval)
{
	ncamwrite( tNAF++, NO_PANTS(Dval));
	ncamwrite( tNAF  ,  TOPLESS(Dval));
}

inline int camwrite(unsigned int N,unsigned int A, unsigned int F,ushort D)
{
	reg_t   cm_s_t  *cm_s_regs= cm_s_alloc(N);
	BASE_TRACE(("CM: camwrite: addr (%d, %d, %d)\n", N,A,F));

	if (POKE(UNAF(N,A,F), D) < 0)
	    BASE_TRACE(("CM: BUS ERROR "));
	return(*CCSR);
}

inline int camread(unsigned int N,unsigned int A,unsigned int F,ushort *D)
{
	reg_t   cm_s_t  *cm_s_regs= cm_s_alloc(N);
	int              val;

	BASE_TRACE(("CM: camread: addr (%d, %d, %d)\n", N,A,F));
	if ((val = PEEK(UNAF(N,A,F))) < 0)
	    BASE_TRACE(("CM: BUS ERROR "));
	*D = TOPLESS(val);
	return(*CCSR);
}

/*
@@@ ================================
@@@	driver entry points:
@@@ cm_open   ( struct inode * node struct file *file)
@@@ cm_release( struct inode * node struct file *file)
@@@ cm_ioctl  ( struct inode * node,struct file *file,
@@@			unsigned int cmd, unsigned long arg)
*/

/* -----------------------------------*/
static int cm_open(struct inode *node,struct file *file)
{
	int err;
	/* dev 0 is opened by a driver and kept during it's presence */
	if(SLOT(node)  == 0)
		return(0);
	if((err = cm_sl_open(SLOT(node),0)) == 0)
	{
		/*
		MOD_INC_USE_COUNT; /**/
		return 0;
	}
	return err;
} /* end CM_OPEN */

static int cm_sl_open(int cminor, int flag)
{
	cm_s_t	*cm_s_regs	= cm_s_alloc(cminor);
	if ((Cm_r_db[CRATE(cminor)] != CRATE_UP) && (flag != START_CRATE))
	    return(ENODEV);
	BASE_TRACE(("CM: cm_sl_open cminor %d , flag %d\n",cminor,flag));
	switch(cm_s_regs->so_status)
	{
        case    SLOT_OPENED:
		return(0);
        case    SLOT_CLOSED:
		if((cm_s_regs->so_ptr = (char *)cm_vme_alloc(cminor))
                                     == (char *)0)
		    return(ENOMEM);
                cm_s_regs->so_status    = SLOT_OPENED;
                cm_s_regs->so_flags     = flag;
                return(0);
        default:
		mess_log(
	"CAM cm_sl_open: unknown opening status of driver %d %x\n"
			,cm_s_regs->so_status
                        ,cm_s_regs->so_status
			);
                return(EFAULT);
	}
	return(0);
} /* end CM_SL_OPEN */

/* -----------------------------------*/
static void cm_release(struct inode *node,struct file *file) 
{
	cm_close(SLOT(node));
	/*
	MOD_DEC_USE_COUNT;
	/**/
} /* end CM_RELEASE */
static void cm_close(int cminor)
{
	reg_t   int     clam;
	reg_t   cm_s_t *cm_s_regs       = cm_s_alloc(cminor);
	reg_t   cm_i_t *cm_i_regs;

	switch(cm_s_regs->so_status)
	{
        case    SLOT_CLOSED:
                break;
        case    SLOT_OPENED:
                /* keep unit 0 for ioctl's */
                if(!cminor)
		    cm_vme_free(cminor);

		cm_s_regs->so_status    = SLOT_CLOSED;
                cm_s_regs->so_ptr       = 0;

		/* Clear all interrupt administration on close (and shut)   */
		for(clam = 1; clam <= 24; clam++)
		{
		    cm_i_regs = cm_i_alloc(clam);
		    if (cm_i_regs->si_cminor == cminor)
		    {
			cm_i_regs->si_procp = (struct proc *)0;
			cm_i_regs->si_pid   = 0;
		    }
		} /* end for */
                break;  
        default:
                break;   
        } /* end switch */
	return;
} /* end CM_CLOSE */

static int cm_ioctl(struct inode *node,struct file *file,
			unsigned int cmd, unsigned long addr)
{
	int 		size,err;
	static unicam_t	u;
        reg_t   int     cminor  = SLOT(node);

	size = _IOC_SIZE(cmd);
	if(size)
	{
		/* we use mostry rw structs  and IOWR */
		err = verify_area(VERIFY_READ , (void *)addr ,size);
		if(err)
			return(err);
		err = verify_area(VERIFY_WRITE, (void *)addr ,size);
		if(err)
			return(err);
		Vm_copy_from_user((char *)&u,(char *)addr,size);
	}
	switch(cmd)
	{
	/* old unix functions are preserved, hence return(-err) !!!! */
	case    CIOCM:
		if((err = camac(cminor, (caddr_t)&u)))
			return(-err);
		break;
	case    CIORTI:
		if(!suser())
			return(-EPERM);
		if((err = lam_allow(cminor)))
			return(-err);
		break;
	case    CIORES:
		if((err = cm_restart(cminor, (caddr_t)&u)))
			return(-err);
		break;
	case    CIOSHUT:
		if((err = cm_shut   (cminor, (caddr_t)&u)))
			return(-err);
		break;
	case    CIOGETCRATE:
		if((err = cm_getcrate(cminor, (caddr_t)&u)))
			return(-err);
		break;
        case    CIOSETINTR:
		if(!suser())
			return(-EPERM);
		if((err = lam_init(cminor, (caddr_t)&u, 0)))
			return(-err);
		break;
        case    CIOGETINTR:
		if(!suser())
			return(-EPERM);
		if((err = lam_init(cminor, (caddr_t)&u, 1)))
			return(-err);
		break;
	/* end set_later */
	default:
		mess_log("CAM cm_ioctl: function %x unknown\n", cmd);
                return (-EINVAL);
        } /* end of switch (cmd)        */
	if(size)
	{
		Vm_copy_to_user((char *)addr,(char *)&u, size);
	}
	return(0);
} /* end CMIOCTL */

/*
 * INTERRUPT ------------------------------------
 * here is interrupt handler, attached in init_mod
 * we try to make a good service routine out of him
 * we use goto as recommended by gnu ;-)))))))))))
 */
/**/
static void cm_poll(int cam_irq,void *dev_id,struct pt_regs *regs)
{
	reg_t   cm_i_t *cm_i_regs;
	reg_t   int     a_level, a_amd, a_pos;
                uint    oldstatus;
	reg_t   short  *tD;
	reg_t   short  *tNAF;
        reg_t   int     clam;
                uint    last_gl;
#ifdef set_later
        /* read GL */
	tD   = (short *)&last_gl;
	tNAF = (short *)&(Cm_regs->gl);

      START_CRITICAL(oldstatus);
/*
** Failure situation:
** Macro ncam24read(tNAF,tD) was causing system crashes when  no  CAMAC  crate
** was  active  (  gl  register feature of CES module).  Appears only when the
** other VME driver is referring to interrupts  by  poll  daisy  chain,  being
** loaded behind CAMAC driver ( later ).
**
** Remedy:
** First access made by this macro  was  replaced  by  peek  ,  escaping  from
** interrupt  on failure.  On success the second short access is made as usual
** by ncamread.  CES is 16 bit module, so do NOT use peekl.
*/

#ifdef  NOT_DEFINED
	ncam24read(tNAF, tD);   /* Replaced by: */
#endif  NOT_DEFINED
	if  ((*tD  = peek(tNAF)) < 0 )
	{
	    END_CRITICAL(oldstatus);
	    return(0);
	}
	(void)ncamread(++tNAF,++tD);

	if  (!last_gl)
	{
	    END_CRITICAL(oldstatus);
	    return(0);
	}

	/* loop to check from which crate/slot interrupt */
	for (clam = 0; clam < 24; clam++)
        {
	    if  ((last_gl & (1 << clam )) == 0)
		continue;
	    clam++;
	    a_level = _amd_intr_lev(clam);
	    a_amd   = _amd_pos(clam);
	    /* Here ev continue if database IMR is already set  */
	    a_pos   = 2 * a_amd + 1;

	    /* disable interrupts from this unit                */
	    Cm_regs->amd_ctrl[a_pos] = BSET_IMR | a_level;
	    Cm_regs->amd_ctrl[a_pos] = BCLR_ISR | a_level;

	    cm_i_regs = cm_i_alloc(clam);
	    /* signal the source to let it clear the mask */
	    if  (cm_i_regs->si_procp != (struct proc *)0)
		psignal(cm_i_regs->si_procp, SIGCAM);
	    else
		mess_log("CAM cm_poll: lam for slot %d not serviced\n", clam);
	    --clam;
	} /* end for */
      END_CRITICAL(oldstatus);
#endif /* set_later */
	return;
} /* end CM_POLL */

/*
@@@ ================================
@@@	module loader wrapper
@@@     init_module(void)
@@@     unload(vdp, vdi)
@@@
*/

/* -----------------------------------*/
static struct file_operations cm_ops = {
	NULL,			/* lseek	*/
	NULL,			/* read 	*/
	NULL,			/* write 	*/
	NULL,
	NULL,
	cm_ioctl,		/* ioctl	*/
	NULL,			/* mmap		*/
	cm_open,
	cm_release,		/* close	*/
	NULL,
	NULL,
	NULL,
	NULL,
};

/* -----------------------------------*/
int init_module(void)
{
	if(_init_cam() < 0)
	    return(EINVAL);
	Cm_attached = 1;
	
	/* I do not know yet how to tackle interrupt shared by two drivers
	** in Linux, so here I do not request interrupt
	if(request_irq(cam_irq,cm_poll,SA_INTERRUPT,Cam_name,NULL))
	{
	BASE_TRACE(("CM: %s: interrupt registration fault on level %d\n",Cam_name,cam_irq));
		return(-ENODEV);
	}
	/**/
	/* 
	 *  funny ways with the dynamic assignment of minor nrs 
	 *  when BIG_MINOR =0 
	 */
	if((cam_major= register_chrdev(CAM_MAJOR,Cam_name,&cm_ops   )) < 0) 
	{
		/*  not tackled yet
		free_irq(cam_irq,NULL);
		/**/
		mess_log("CM falied to register CAMAC driver under major %d\n",CAM_MAJOR);
		return -EIO;
	}
	if (CAM_MAJOR > 0)
		cam_major = CAM_MAJOR;
	mess_log("CM: LOAD  major %d ( asked %d)  no LAM irq yet \n"
		,cam_major,CAM_MAJOR);
	return(0);
} /* end INIT_MODULE */

void cleanup_module(void)
{
	reg_t   int crate;
	BASE_TRACE(("CM: UNLOAD cm %d ( %d) /* free intr %d/**/\n"
			,cam_major,CAM_MAJOR,cam_irq));
	/* not tackled yet 
	free_irq(cam_irq,NULL); /**/
	if(cam_major > 0)
		unregister_chrdev(cam_major,Cam_name);
	else
		unregister_chrdev(CAM_MAJOR,Cam_name);
	/* CAMAC */
	if(!Cm_attached)
	    return;

	for(crate = 1;crate <= MAX_CR_CTRL;crate++)
	    shut_crate(crate);
	
	/* shut all existing mappings */
	/* shut register page	*/
	cm_vme_free(SLOT_CES);

	/* kill all processes using interrupt	*/

	/* Decouple interrupt on SUNs:
	** Can be a problem with other (not yet) VME drivers.
	** When no remintr is done everything works, but when installing a new
	** driver it will panic in addioname on LOAD of the new one.
	** Panic on:
	**   &cm_poll has a differend value and the interrupt was not removed
	*/

} /* end CM_UNLOAD */

/*
@@@ ================================
@@@ camac init routine
@@@     _init_cam() - returns -1 on failure
@@e
*/
	static int
_init_cam()
{
	reg_t   int     crate;
		int     btb_bz;
 	/* checking the vmehb driver for presence */                
	mess_log("CAM init: starting CAMAC driver camces\n");	
	/* Probing of CES module, init CCSR, fire BZ, start crates      */
	for(crate = 1; crate <= MAX_CR_CTRL; crate++)
	    Cm_r_db[crate] = CRATE_DOWN;

	if  ((Cm_regs = (cm_t *)cm_vme_alloc(SLOT_CES)) == (cm_t *)0)
        {
	    mess_log("CAM init: resources allocation error\n");
	    return(-1);
        }

	if(peek(CCSR) < 0)
        {
	    mess_log("CAM no CES module found\n");
	    return(-1);
        }

	ncamwrite(CCSR, (CCSR_DEF | MLAM));

	if(poke(&(Cm_regs->btb_bz), 0))
	{
	    mess_log("CAM init: cannot write CES board (BRANCH_Z)\n");
	    return(-1);
	}
	DELAY(100);
	Cm_regs->btb_bz = 0;
	DELAY(100);

	_am_vec_init();

	ncamread (&(Cm_regs->btb_bz), (ushort *)&btb_bz);
        if(btb_bz == 0)
	    mess_log("CAM crates off line\n");
        else
	    for(crate = 1; crate <= MAX_CR_CTRL; crate++)
		start_crate(crate);
	mess_log("CAM init : initiated\n");
	return(0);
} /* end _INIT_CAM */

/*
@@@ ================================
@@@ ioctl executing routine for base access of a CAMAC register
@@@
*/

int camac( int cminor, caddr_t addr)
{
	reg_t   camcall_t      *uap             = (camcall_t *)addr;
	reg_t   cm_s_t         *cm_s_regs       = cm_s_alloc(cminor);
        reg_t   int             camfun;
	reg_t   short          *tNAF;
	reg_t   short          *tD;
		ushort          btb_bz;
                ushort          camd;
                uint            oldstatus;
 
        /* preset ccsr for faulty return */
        uap->cm_ccsr = CCSR_ERR;

        /* check parameters */
	if(Cm_r_db[CRATE(cminor)] != CRATE_UP)
		return(ENODEV);
	if(cm_s_regs->so_status   != SLOT_OPENED)
                return(ENODEV);
        if(cm_s_regs->so_ptr  == 0)
                return(ENOMEM);
        if((uap->cm_a & N_AMASK) || ( uap->cm_f & N_FMASK))
                return(EINVAL);

	/* Check on line status and shut crate when not online */
	ncamread (&(Cm_regs->btb_bz), &btb_bz);
	if (!(btb_bz & (1 << CRATE(cminor))))
	{
	    mess_log("CAM crate %2d: not online while camac (warning)\n", CRATE(cminor));
	    mess_log("CAM crate %2d: disabled by force\n", CRATE(cminor));
	    Cm_r_db[CRATE(cminor)] = CRATE_DOWN;
	    return(ENODEV);
	}

        camfun = uap->cm_f;

	switch(cm_action(camfun, uap->cm_l))
	{
        case    (CAM_RD|CAM_16): /* also = (CAM_EX) */
		START_CRITICAL(oldstatus);
		  uap->cm_ccsr = camread(cminor, uap->cm_a, camfun, &camd);
		  uap->cm_data = (unsigned int)camd;
		END_CRITICAL(oldstatus);
		return(0);
        case    (CAM_WR|CAM_16):
		START_CRITICAL(oldstatus);
		  uap->cm_ccsr = camwrite(cminor, uap->cm_a, camfun, uap->cm_data & 0xffff);
		END_CRITICAL(oldstatus);
                return(0);
        case    CAM_RD:
		tNAF    = (short *)NAFA(cminor, uap->cm_a, camfun);
                tD      = (short *)&uap->cm_data;

		START_CRITICAL(oldstatus);
		  ncam24read(tNAF, tD);
		  uap->cm_ccsr = *CCSR;
		END_CRITICAL(oldstatus);
                return(0);
        case    CAM_WR:
		tNAF= (short *)NAFA(cminor, uap->cm_a, camfun);

		START_CRITICAL(oldstatus);
		  ncam24write(tNAF, uap->cm_data);
		  uap->cm_ccsr = *CCSR;
		END_CRITICAL(oldstatus);
                return(0);
        default:
		mess_log("CAM camac: bad type of action \n");
                return(EINVAL);
        }
} /* end CAMAC */

/* 
@@@ ================================
@@@ ioctl executing routines of interrupt + init routine
@@@ 	Lam sources are allowed to one-user/one-slot in all crates!!!
@@@	because of 24 sources to be distinguished by GRADED LAM
@@@
@@@	lam_allow( cminor)		- unmask lam interrupt
@@@	lam_ignore(cminor)		-   mask lam interrupt ( for symmetry)
@@@     lam_init(  cminor, addr,flag)   - init administration of lam's
@@@	_am_vec_init()			- void initializer of CES interrupt chip
@@@
*/

int lam_allow(ushort cminor)
{
	reg_t   int     a_level, a_amd, a_pos;

        a_level = _amd_intr_lev(cminor);
        a_amd   = _amd_pos(cminor);
        a_pos   = 2 * a_amd + 1;
	Cm_regs->amd_ctrl[a_pos] = BCLR_IRR_IMR | a_level;
        return(0);
} /* end LAM_ALLOW */

int lam_ignore(ushort cminor)
{
                uint    oldstatus;
	reg_t   int     a_level, a_amd, a_pos;
 
        a_level = _amd_intr_lev(cminor);
        a_amd   = _amd_pos(cminor);
        a_pos   = 2 * a_amd + 1;

        START_CRITICAL(oldstatus);
	  /* disable interrupts from this unit  */
	  Cm_regs->amd_ctrl[a_pos] = BSET_IMR | a_level;
	  Cm_regs->amd_ctrl[a_pos] = BCLR_ISR | a_level;
        END_CRITICAL(oldstatus);

        return(0);
} /* end LAM_IGNORE */

int lam_init(ushort cminor, caddr_t addr,int  flag)
{
#define LAM(slot)       (N_CR(slot))
        reg_t   int             l_pos           = LAM(cminor);
	reg_t   intcall_t      *uap             = (intcall_t *)addr;
	reg_t   cm_i_t         *cm_i_regs       = cm_i_alloc(l_pos);
	reg_t   cm_s_t         *cm_s_regs       = cm_s_alloc(cminor);

        /* check parameters */
	if(Cm_r_db[CRATE(cminor)] != CRATE_UP)
		return(ENODEV);
	if(cm_s_regs->so_status != SLOT_OPENED)
                return(ENODEV);
        if(cm_s_regs->so_ptr  == 0)
                return(ENOMEM);

	switch (flag)
	{
        case    0:      /* lam set /clr */
                cm_i_regs->si_cminor    = cminor;
		cm_i_regs->si_pid       = uap->in_pid;
                if(cm_i_regs->si_pid)
                {
		    /* cm_i_regs->si_procp = u.u_procp; /* set later */
		    lam_allow(cminor);  /* set */
                }
                else
                {
		    lam_ignore(cminor); /* clr */
		    cm_i_regs->si_procp = (struct proc *)0;
                }
                break;
        case    1:      /* lam get */
		uap->in_cminor  = cminor;
		uap->in_crate   = CRATE(cminor);
		uap->in_pid     = cm_i_regs->si_pid;
                break;
        default:
                return(EINVAL);
	} /* end switch (flag)  */
        return(0);
} /* end LAM_INIT */

void _am_vec_init()
{
	reg_t   int     i, j, k, l;

        k = CM_GROUP_VECTOR ;
        for(i = 0;i < 3;i++)
        {
	    /* addressing of reg_ts */
	    j = 2 * i + 1;
	    Cm_regs->amd_ctrl[j] = RESET_AMD;
	    /*
	    ** ROUND_ROBIN PRIORITY 1 ( FIXED 0)
	    ** GROUP VECTORING      2 ( ALWAYS FOR THIS VERSION )
	    ** INTERRUPT MODE: GINT LOW IREQ LOW
	    */
	    Cm_regs->amd_ctrl[j] = MOD_LO & 3;
	    /* set  group vector */
	    for( l = 0;l < 8;l++)
	    {
		Cm_regs->amd_ctrl[j] = WR_MEM(1, l);
		Cm_regs->amd_data[j] = k;
	    }
	    /* enable chip                  */
	    Cm_regs->amd_ctrl[j] = 0xa1;
        }
} /* end _AM_VEC_INIT */

/* 
@@@ ================================
@@@ ioctl executing routines of crate management and subordinates
@@@	Routines used for reconfigurtion of crate and restarting
@@@     cm_restart(cminor, addr)
@@@     cm_shut   (cminor, addr)
@@@
*/

int cm_restart(int cminor,caddr_t addr)
{
	reg_t   int     crate;
		int	err;

	if  ((cminor != 0) || (addr == 0))
	{
	    return(ENXIO);
	}
	crate   = ((cracall_t *)addr)->cr_crate;

	if  (crate <= 0 || crate > MAX_CR_CTRL)
	{
	    mess_log("CM cm_restart: wrong crate: %d\n", crate);
	    return(EINVAL);
	}
	err = start_crate(crate);
	return(err);
} /* end CM_RESTART */

int cm_shut(int cminor,caddr_t addr)
{
	reg_t   int     crate;

	BASE_TRACE(("CM: shut cminor %x addr %x\n",cminor,(int)addr));
	if  ((cminor != 0) || (addr == 0))
	    return(ENXIO);
	crate   = ((cracall_t *)addr)->cr_crate;

	if  (crate <= 0 || crate > MAX_CR_CTRL)
	{
	    mess_log("CM cm_shut: wrong crate: %d\n", crate);
	    return(EINVAL);
	}
	return(shut_crate(crate));
} /* end CM_SHUT */


int start_crate(unsigned int crate)
{
	int             func, status, try;
	cm_s_t         *cm_s_regs;
	ushort          camd;
	ushort          btb_bz;
	uint            oldstatus;

	if ((Cm_r_db[crate] == CRATE_UP))
	    return(EALREADY);

	ncamread (&(Cm_regs->btb_bz), &btb_bz);
	if (btb_bz)
	{
	    if (!(btb_bz & (1 << crate)))
	    {
		mess_log("CAM crate %2d: not online\n", crate);
		return(ENETUNREACH);
	    }
	}
	else /* btb_bz == 0  */
	{
	    mess_log("CAM none of the crates are online\n");
	    return(ENETDOWN);
	}

	/* Z on crate */
	cm_s_regs = cm_s_alloc(N_EXT(SLOT_Z, crate));
	if  (cm_sl_open(N_EXT(SLOT_Z, crate), START_CRATE) != 0)
	{
	    mess_log("CAM start_crate: cannot open device %x for Z\n", N_EXT(SLOT_Z, crate));
	    return(EFAULT);
	}

	if  (peek(UNAF(N_EXT(SLOT_Z, crate), 8, 26)) < 0)
	{
	    mess_log("CAM crate %2d: timeout on Z (First probe)\n", crate);
	    cm_close(N_EXT(SLOT_Z, crate));
	    return(EFAULT);
	}

	START_CRITICAL(oldstatus);
	  status = camread(N_EXT(SLOT_Z, crate), 8, 26, &camd);
	END_CRITICAL(oldstatus);

	if(NO_X_RESPONSE(status))
	    mess_log("CAM crate %2d: no X response on Z (warning)\n", crate);

	cm_close(N_EXT(SLOT_Z, crate));

	/*
	** Remove Dataway Inhibit (e.g.for TDC's)
	** Enable BD on crate controller(s)
	*/
	cm_s_regs = cm_s_alloc(N_EXT(SLOT_CTRL, crate));
	if  (cm_sl_open(N_EXT(SLOT_CTRL, crate), START_CRATE) != 0)
	{
	    mess_log("CAM start_crate: cannot open device %x for SLOT_CTRL\n",
		     N_EXT(SLOT_CTRL, crate));
	    return(EFAULT);
	}

	DELAY(1000);  /* micro seconds busy wait    after Z         */
	for (try=0; try<MAXTRY; try++)
	{                                   /* clr Dataway Inhibit  */
	    camread(N_EXT(SLOT_CTRL, crate), 9, 24, &camd);
	    START_CRITICAL(oldstatus);      /* check DW inhibit     */
	      status = camread(N_EXT(SLOT_CTRL, crate), 9, 27, &camd);
	    END_CRITICAL(oldstatus);
	    if(NO_Q_RESPONSE(status))       break;
	}
	if (try==MAXTRY)
	    mess_log("CAM crate %2d: enabled DW inhibit(warning)\n", crate);
#ifdef  NOT_DEFINED
	/* Set GL Mask Register (24bits) (Only A2 Controllers) */
	START_CRITICAL(oldstatus);
	  status = cam24write(N_EXT(SLOT_CTRL, crate), 13, 17, 0xffffffL);
	END_CRITICAL(oldstatus);

	if    (NO_X_RESPONSE(status))
	    mess_log("CAM no X response on SET GL MASK REGISTER (warning)\n");
	else  if(NO_Q_RESPONSE(status))
	    mess_log("CAM no Q response on SET GL MASK REGISTER (warning)\n");
#endif  NOT_DEFINED

	for (try=0; try<MAXTRY; try++)
	{                                   /* set BD output        */
	    camread(N_EXT(SLOT_CTRL, crate), 10, 26, &camd);
	    START_CRITICAL(oldstatus);      /* check BD output      */
	      status = camread(N_EXT(SLOT_CTRL, crate), 10, 27, &camd);
	    END_CRITICAL(oldstatus);
	    if(Q_RESPONSE(status))  break;
	}

	if (try==MAXTRY)
	    mess_log("CAM crate %2d: disabled BD output (warning)\n", crate);

	cm_close(N_EXT(SLOT_CTRL, crate));
	Cm_r_db[crate] = CRATE_UP;
	/* ncamwrite(CCSR, (CCSR_DEF | MLAM));  /* Debug */
	ncamwrite(CCSR, CCSR_DEF);
	mess_log ("CAM crate %2d:  enabled\n", crate);
	return(0);
} /* end START_CRATE */

int shut_crate(unsigned int crate)
{
	ushort          camd;
	ushort          btb_bz;
	reg_t		cm_p_t	*cm_p_regs;
	reg_t		int	cminor;
	reg_t		int	i;

	if (Cm_r_db[crate] == CRATE_DOWN)
	    return(EALREADY);

#define BD_DISABLE
#ifdef  BD_DISABLE
	ncamread (&(Cm_regs->btb_bz), &btb_bz);
	if (!(btb_bz & (1 << crate)))
	{
	    mess_log("CAM crate %2d: not online while shut_crate (warning)\n", crate);
	    mess_log("CAM crate %2d: disabled\n", crate);
	    Cm_r_db[crate] = CRATE_DOWN;
	    return(0);
	}

	if  (cm_sl_open(N_EXT(SLOT_CTRL, crate), START_CRATE) != 0)
	{
	    mess_log("CAM shut_crate: cannot open device %x for SLOT_CTRL\n",
		     N_EXT(SLOT_CTRL, crate));
	    return(EFAULT);
	}
	camread (N_EXT(SLOT_CTRL, crate), 10, 24, &camd); /* Disable BD */
	cm_close(N_EXT(SLOT_CTRL, crate));
	mess_log("CAM crate %2d: disabled (Also BD)\n", crate);
#else   BD_DISABLE
	mess_log("CAM crate %2d: disabled\n", crate);
#endif  BD_DISABLE

	for(i = 1;i < 32;i++)
	{
		cminor		= N_EXT(i,crate);
		cm_p_regs	= cm_p_alloc(cminor);
		if(cm_p_regs->pa_ptr != NULL)
		{
			/* this forces unmap to perform */
			cm_p_regs->pa_cnt = 0;
			cm_vme_free(cminor);
		}
	}

	Cm_r_db[crate] = CRATE_DOWN;
	return(0);
} /* end SHUT_CRATE */

int cm_getcrate(int cminor, caddr_t addr)
{
        reg_t   int             crate;
	reg_t   int             func;

        /* setup and check parameters */
	if ((cminor != 0) || (addr == 0))
	    return(ENXIO);

	crate   = ((cracall_t *)addr)->cr_crate;
	func    = ((cracall_t *)addr)->cr_func;    /* Always 2 yet */

	if (crate < 1 || crate > MAX_CR_CTRL ) 
	{
	    mess_log("CAM cm_getcrate: args crate %d func %d\n", crate, func);
	    return(EINVAL);
	}

	((cracall_t *)addr)->cr_status = Cm_r_db[crate];
	return(0);
} /* end CM_GETCRATE */

/* 
@@@ ================================
@@@ memory allocation routines
@@@     cm_vme_alloc(cminor)
@@@     cm_vme_free (cminor)
*/

int cm_vme_alloc(int cminor)
{
        reg_t   cm_p_t  *cm_p_regs      = cm_p_alloc(cminor);
	reg_t   int      cminimus       = cminor & 0x3;
	int	temporary;

        /* check if everything already done     */
	if(cm_p_regs->pa_kmx)
        {
	    cm_p_regs->pa_cnt++;
	    return(((int)cm_p_regs->pa_ptr) + (cminimus * SLOT_SIZE));
        }

#ifdef	linux
	/* Here we have to fiddle with vmehb mapping */
	cm_p_regs->pa_ptr = (caddr_t)vmehb_dev_mmap(VME_MINOR,HARD_ADDR(cminor),0x2000);
	temporary = HARD_ADDR(cminor);
	BASE_TRACE(("CM: VME %8x mapped at: %x\n"
		,temporary,cm_p_regs->pa_ptr));
#endif /* linux */

#ifndef linux /* old hardware dependencies for the history */
#ifdef	sun4e
	cm_p_regs->pa_ptr = (char *)map_regs(HARD_ADDR(cminor), 0x2000, 2);
#else   
#ifdef  PTVME_DRIVER
	{
	  extern struct pt_softc *pt_softcp;
	  cm_p_regs->pa_ptr = (char *)vme_map(pt_softcp, HARD_ADDR(cminor), 0x2000, 0x3d);
	}
#else
	cm_p_regs->pa_ptr = (char *)map_regs(HARD_ADDR(cminor), 0x2000, 1);
#endif
#endif
#endif /* no linux */
	if(cm_p_regs->pa_ptr == NULL)
	{
	    mess_log("CAM cm_vme_alloc: no free pages\n");
	    return(0);
	}
	cm_p_regs->pa_kmx = 1;
	cm_p_regs->pa_cnt++;

	return(((int)cm_p_regs->pa_ptr) + (cminimus * SLOT_SIZE)) ;
} /* end CM_VME_ALLOC */

int cm_vme_free(int cminor)
{
        reg_t   cm_p_t  *cm_p_regs      = cm_p_alloc(cminor);

	/* do not close page if other slots active      */
	if((--(cm_p_regs->pa_cnt)) > 0 )
	    return;
#ifdef	linux
	cm_p_regs->pa_ptr = (char *)vmehb_dev_unmap(VME_MINOR,HARD_ADDR(cminor),0x2000);
#endif

#ifndef linux
#ifdef  PTVME_DRIVER
	vme_unmap(pt_softcp,  HARD_ADDR(cminor), 0x2000, 0x3d);
#else
	unmap_regs(cm_p_regs->pa_ptr,0x2000);
#endif
#endif
	cm_p_regs->pa_cnt = 0;
	cm_p_regs->pa_ptr = 0;
	cm_p_regs->pa_kmx = 0;
} /* end CM_VME_FREE */


