
#ifndef	__CAMCES_CONF_H
#define	__CAMCES_CONF_H
/*
    camces - driver of VME device CMD 8210 of CES
             for CAMAC bus parallel branch driving 
    ----------------------------------------------------------------------
    driver configuration file 
    The file covers the following aspects:
    	Hardware dependency:
    		current configuration
    	System dependency:
    		including proper system files
   		system independent wrappers for routines & macros
    	System tracing and debugging:
    This file contains VARIABLES !!!!

    Copyright (C) May 1997  Natalia Kruszynska <natalia@nikhef.nl>
	Thanks for contribution to:
		Ruud van Wijk   <ruud@nikhef.nl>
		Marcin Dalecki  <dalecki@sub944.sub.uni-goettingen.de>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* ================================================================== 
 * The following  prefixes are consequently used in the code:
 *	Vm	- wrappers for system /version dependent functionality
 *	cm_	- syscalls of a driver
 *	(n)cam	- access inline functions
 */

/* ================================================================== 
 * System include files
 */

#	include	<linux/module.h>
#	include	<linux/wrapper.h>
#include	<linux/sched.h>		/* for resources routine */
#include	<linux/kernel.h>
#include	<linux/malloc.h>
#include	<linux/types.h>
#include	<linux/ioport.h>	/* for port operations	*/
#include	<linux/fs.h>
#include	<linux/mm.h>
#include	<linux/errno.h>
#include	<asm/io.h>		/* for port operations	*/
#include	<linux/bios32.h>	/* for pci_bios & probe */
#include	<linux/pci.h>
#include	<asm/segment.h>  	/* for get/put into user space	*/
#include	<asm/irq.h>		 /* for interrupt	*/
#include	<linux/delay.h>		 /* for udelay(usecs)	*/
#include	<linux/wait.h>		 /* for semaphoring	*/
#include	<linux/smp.h>		 /* for semaphoring	*/

/* ==================================================================
 * local include files 
 */
#include	"camdef.h"
#include	"cam.h"
#include	"camioctl.h"
/* ==================================================================
 *  SYSTEM COMPATIBILITY   in both Linux lines 
 *	Kernel/user copying in mem & i/o:
 *      	Vm_copy..io	- for read/write
 *      	Vm_copy..user   - for ioctl
 */
#ifdef LIN2_0 /* in 2_0_0 and 2_0_30 is the same */
#define	Vm_copy_to_user		memcpy_tofs
#define	Vm_copy_from_user	memcpy_fromfs
#define	Vm_copy_fromio		memcpy_tofs
#define	Vm_copy_toio		memcpy_fromfs
#define	Vm_get_uval(a,b)  	(a) = get_user((b))
#define	Vm_put_uval(a,b)  	put_user((a),(b))
#endif
#ifdef LIN2_1_27
/* changes from 2.0.0 to 2.1.27					*/
#include	<asm/uaccess.h>		/* VERIFY_READ/WRITE flags are here */
#define	vremap			ioremap
#define vfree			iounmap
/* see whether it works */
#define	Vm_copy_to_user		memcpy_fromio
#define	Vm_copy_from_user	memcpy_toio
#define	Vm_copy_fromio		memcpy_fromio
#define	Vm_copy_toio		memcpy_toio
#define	Vm_get_uval(a,b) 	get_user((a),(b))
#define	Vm_put_uval(a,b) 	put_user((a),(b))
#endif
/* ================================================================== 
 * DEBUGGING - more levels can be added besides BASE_DEBUG
 */
#define BASE_DEBUG			/* debugging primitives	*/
/* fine SYS_TRACELEN		0x200	/* tracing length 	*/

/* do not change this : 					*/
#ifdef	BASE_DEBUG
#	define	BASE_TRACE(x)	printk x
#else
#	define	BASE_TRACE(x)	
#endif


/* ==================================================================
 * CONFIGURATION CONSTANTS:
 */

/* ================================================================== 
 * INLINE wrappers for system/version dependent stuff - all here !!! 
 */

/*  Semaphores */
/*  ---------- */

static  inline void Vm_down(struct semaphore *sem)
{
	down(sem);
} /* end Vm_down() */
static inline void Vm_up(struct semaphore *sem)
{
	up(sem);
} /* end Vm_up() */
/* ================================================================== 
 *   Driver flags:
 */

/* ================================================================== 
 * SYSTEM TRACING: 
 * reported on interrupty and by the extra ioctl
 * You can see how to use and how to add. Try it once...
 */
#ifdef SYS_TRACELEN
static	int 	trace[SYS_TRACELEN];
static	int	traceno 	= 0;
/* TRACING A - not used yet		 */
/* TRACING B - not used yet		 */
/* TRACING C - not used yet              */
static	int	count0		= 0;
static	int	count1		= 0;
static	int	count2		= 0;
#define	more_0		++count0
#define	more_1		++count1
#define	more_2		++count2
#define	set_C(val)	
#define	set_B(val)	
/* Here we set the actual level of tracing to A */
static  inline  set_A(val)	{
	trace[traceno++] = (unsigned int)val;
	if(traceno > SYS_TRACELEN - 1)
		traceno = 0;
};
static void print_trace()
{
	int i;
	printk("Current trace : %x\n",traceno);
	for(i = 0;i < SYS_TRACELEN;i+= 8)
	{
		printk("%3x: %8x %8x %8x %8x|%8x %8x %8x %8x\n"
			,i
			,trace[i]
			,trace[i + 1]
			,trace[i + 2]
			,trace[i + 3]
			,trace[i + 4]
			,trace[i + 5]
			,trace[i + 6]
			,trace[i + 7]
			);
	}
}
#else
#define	set_A(val)	
#define	set_B(val)	
#define	set_C(val)	
#define	print_trace()
#define	more_0	
#define	more_1
#define	more_2
#endif

#endif 	/* __CAMCES_CONF_H */
