/*
** CAM_START - CMRESTART, CMSHUTDOWN, CMGETCRATE
*/
# include	<sys/types.h>
# include	<sys/ioctl.h>
# include       <stdio.h>
# include       "camioctl.h"
# include	"camac.h"

camrest (int crate, int retry)
{
	reg_t	int		status;
		cracall_t       ua;
	reg_t   cracall_t      *uap = { &ua };
		int             fd;

	uap->cr_crate   = crate;
	uap->cr_func    = 0;
	if ( (fd = open(Cam_sysdev, 0)) == -1)
	{
	    if(Cam_verbose != CAM_SILENT)
		perror("camrest: cannot open system device");
	    return(-1);
	}
	if(retry > 8)
	    retry = 8;
again:
	status = ioctl(fd, CIORES, uap);
	if(status < 0)
	{
	    if(Cam_verbose != CAM_SILENT)
	    {
		fprintf(stderr,"camrest: crate %d", crate);

		switch(errno)
		{
		case    EALREADY:
			fprintf(stderr," CAMAC already restarted\n");
			return(1);
		case    ENETUNREACH:
			fprintf(stderr," CAMAC crate cannot reset\n");
			break;
		case    ENETDOWN:
			fprintf(stderr," CAMAC net down\n");
			break;
		default:
			perror(" ");
		} /* switch(errno) */
	    } /* end if(Cam... */

	    if((retry--) > 0)
	    {
		fprintf(stderr,"        ...retry\n");
		sleep(8);
		goto again;
	    }
	}
	else
	    if(Cam_verbose != CAM_SILENT)
		fprintf(stderr,"camrest: crate %d restarted\n", crate);
	close(fd);
	return(status);
}

camshut(int crate)
{
	reg_t	int		status;
		cracall_t       ua;
	reg_t   cracall_t      *uap = { &ua };
		int             fd;

	uap->cr_crate   = crate;
	uap->cr_func    = 1;
	if ( (fd = open(Cam_sysdev,0)) == -1)
	{
	    perror("camshut: cannot open system device");
	    return(-1);
	}
	uap->cr_func = 1;
	status = ioctl(fd, CIOSHUT, uap);
	if(status < 0)
	{
	    if(Cam_verbose != CAM_SILENT)
	    {
		fprintf(stderr,"camshut: crate %d ", crate );
		switch(errno)
		{
		case    EALREADY:
			fprintf(stderr," CAMAC already shut\n");
			return(1);
		default:
			perror(" ");
		} /* switch(errno) */
	    } /* end if */
	}
	else
	    if(Cam_verbose != CAM_SILENT)
		fprintf(stderr,"camshut: crate %d shutdown\n", crate);
	close(fd);
	return(status);
}

camcrate(int crate)
{
	reg_t	int		status;
		cracall_t       ua;
	reg_t   cracall_t      *uap = { &ua };
		int             fd;

	if ( (fd = open(Cam_sysdev,0)) == -1)
	{
	    perror("camcrate: cannot open system device");
	    return(-1);
	}

	uap->cr_crate   = crate;
	uap->cr_func    = 2;
	uap->cr_status  = -1;

	status = ioctl(fd, CIOGETCRATE, uap);
	if(status < 0)
	{
	    if(Cam_verbose != CAM_SILENT)
		perror("camcrate: ioctl error");
	    status = -1;
	}
	else
	    status = uap->cr_status;
	close(fd);
	return((status == -1)       ?   -1:
	       (status == CRATE_UP) ?    1: 0);

}
