#include <stdio.h>
#include "cmd.h"
#include "def.h"

       char    Orlist[4*MAXLEN];
extern CMDLIST Global_cmds;

/*
**      CMD_MATCH ------------------------------------------------------------
**      Test if name matches global cmds and/or local cmds and/or a key list.
**      If one or more list pointers are NULL, the lists are not used.
**
**      The result can be one of:                                   return:
**      ------------------------                                    ---------
**      1)  Matches exactly global                                  MATCH_GLO
**      2)  Matches exactly  local                                  MATCH_LOC
**      3)  Matches exactly    key                                  key index
**      4)  Matches leftsub global only ones and no  local or key   MATCH_GLO
**      5)  Matches leftsub  local only ones and no global or key   MATCH_LOC
**      6)  Matches leftsub    key only ones and no global or local key index
**      7)  Matches leftsub more then ones local or key or global   MATCH_AMB
**          In this case also print the ambiguous message.
**      8)  Doesn't match at all.                                   MATCH_NOT
**
**      In case of MATCH_GLO or MATCH_LOC return also via (CMD **)cmdptr.
**      MATCH_GLO/ _LOC/ _NOT/ _AMB are negativ values, key index is positiv.
*/


cmd_match(name, loclist, keylist, cmdptr)
char  * name;
CMDLIST loclist;
KEYLIST keylist;
CMD   **cmdptr; /* Output only parameter */
{
	extern   BOOL   Glob;
	extern   BOOL   leftsub();
	extern   char   Orlist[];
		 int    i, locent, gloent, keyent, loccnt, glocnt, keycnt;

	glocnt = loccnt = keycnt = 0;
	gloent = locent = keyent = 0;

	if (keylist && loclist)
	    error("cmd_match key- and local-list (warning only)");

	/* Look if the name matches completely a global command */
	if (Global_cmds)
	    for (i = 1; Global_cmds[i].name != NULL; ++i)
		if (!strcmp(name, Global_cmds[i].name))
		{
		    *cmdptr = &Global_cmds[i];
		    return(MATCH_GLO);
		}

	/* Look if the name matches completely a  local command */
	if (loclist)
	    for (i = 1; loclist[i].name != NULL; ++i)
		if (!strcmp(name, loclist[i].name))
		{
		    *cmdptr = &loclist[i];
		    return(MATCH_LOC);
		}

	/* Look if the name matches completely a key name */
	if (keylist)
	    for (i = 0; keylist[i] != NULL; ++i)
		if (!strcmp(name, keylist[i]))
		    return (i);

	Orlist[0] = '\0';

	/* Look how many times the name matches partly a global name */
	if (Global_cmds)
	    for (i = 1; Global_cmds[i].name != NULL; ++i)
		if (leftsub(name, Global_cmds[i].name))
		{
		    orname(Global_cmds[i].name);
		    glocnt++;
		    gloent = i;
		}

	/* Look how many times the name matches partly a local name */
	if (loclist)
	    for (i = 1;  loclist[i].name != NULL; ++i)
		if (leftsub(name, loclist[i].name))
		{
		    orname( loclist[i].name);
		    loccnt++;
		    locent = i;
		}

	/* Look how many times the name matches partly a key name */
	if (keylist)
	    for (i = 0; keylist[i] != NULL; ++i)
		if (leftsub(name, keylist[i]))
		{
		    orname( keylist[i]);
		    keycnt++;
		    keyent = i;
		}

	if  (!glocnt && !loccnt && !keycnt)
	    return(MATCH_NOT);                  /* No match at all          */

	if  ((glocnt == 1) && (!loccnt) && (!keycnt))
	{
	    *cmdptr = &Global_cmds[gloent];
	    return (MATCH_GLO);                 /* Only one global          */
	}

	if  ((loccnt == 1) && (!keycnt) && (!glocnt))
	{
	    *cmdptr = &loclist[locent];
	    return (MATCH_LOC);                 /* Only one local           */
	}

	if  ((keycnt == 1) && (!loccnt) && (!glocnt))
	    return (keyent);                    /* Only one key             */

	error("Not unique: %s%sDo you mean %s ?", name, Cerror, Orlist);
	return (MATCH_AMB);
}


orname(name)
char *	name;
{
	extern  char Orlist[];

	if (strlen(Orlist))
	{
	    strcat(Orlist, Cerror);
	    strcat(Orlist, "         or ");
	    strcat(Orlist, name);
	}
	else
	    strcat(Orlist, name);
}
