#include <stdio.h>
#include <setjmp.h>
#include "cmd.h"
#include "def.h"

#define ENV_COPY(to,from)	{ int i; for (i = 0; i < 3; i++) to[i] = from[i]; }

jmp_buf	CmdEnv;

/*
 *	COMMAND ----------------------------------------------------------------
 *	Reads a command from input and compares it with the lists of
 *	legal commands (local_cmds and global_cmds).
 *	If it is a legal command, this is executed. Otherwise, an error
 *	message is given, and the next command is read.
 *	Normally, after the execution of a command, the procedure
 *	mentioned above is repeated. However, when the global variable
 *	"Single_mode" is TRUE, a return is made to the caller.
 *
 *	Return values of command are:
 *	- STOP, indicating the global command "quit" was given;
 *	- HOME, indicating the global command "restart" was given;
 *	- PREV (previous): cannot occur;
 *	- CONT (continue), in all other cases.
 */

	ACTION
command(local_cmds)
CMDLIST	local_cmds;
{
		char	answer[MAXLEN];
		ACTION	status;
		int (*	save_interrupt)();
		jmp_buf	saveEnv;

	extern  BOOL    Single_mode;    /* If TRUE, return after a command. */
	extern  char    Ch;             /* Next input character.            */
	extern  int     Help_type;      /* Type of expected input.          */
	extern  CMD *   Help_list;      /* Commandset for "help".           */
	extern	jmp_buf	CmdEnv;

	save_interrupt = set_interrupt(INT_OFF);
	ENV_COPY(saveEnv,CmdEnv);

	do
	{
	    status = CONT;
	    if (setjmp(CmdEnv) == 1 && Single_mode)
		break;
	    set_interrupt(INT_ON);
	    if (DELIMITER(Ch))
	    {
		if (Ch == '\n' && ! Single_mode)
		{
		    printf("%s%s%s", Bprompt,local_cmds[0].name,Eprompt);
		    fflush(stdout);
		}
		Ch = in_char();
		if (Ch == '!')
		    cmd_shell();
		continue;
	    }
	    if (in_item(answer) == NULL)
	    {
		status = STOP;
		break;
	    }
	    lower_case(answer);

	    {
		int  ret;
		CMD *cmdptr;

		ret = cmd_match(answer, local_cmds, NULL, &cmdptr);
		switch(ret)
		{
		case MATCH_AMB:
		    break;
		case MATCH_NOT:
		    error("Command not recognized: %s",answer);
		    break;
		case MATCH_LOC:
		    status = (cmdptr->function)(cmdptr->name);
		    break;
		case MATCH_GLO:
		    {
			BOOL    save_mode;
			save_mode = Single_mode; Single_mode = FALSE;
			Help_type = H_COMMAND; Help_list = local_cmds;
			status = (cmdptr->function)(cmdptr->name);
			Single_mode = save_mode;
		    }
		    break;
		} /* end switch (ret ... */
	    } /* end module */
	} while (status == CONT && ! (Single_mode && Ch == '\n'));

	ENV_COPY(CmdEnv,saveEnv);
	set_interrupt(save_interrupt);
	return ((status == PREV) && ! Single_mode ? CONT : status);
}
