#include <stdio.h>
#include <signal.h>
#include <errno.h>
#include "cmd.h"
#include "def.h"

int	Help_type;	/* type of expected input */
char *	Help_text;	/* description of expected parameter type */
char *	Help_file;	/* name of file with extended parameter description */
CMD *	Help_list;	/*   commandset */
/*
 *	IN_ITEM ----------------------------------------------------------------
 *	Read next item from input. Item is separated by semicolon, space, tab
 *	or newline. If first character is quote, item continues until next
 *	quote or newline (for safety reasons). Ending delimiter is not included
 *	in item (but starting quote is). On entry, Ch contains first character.
 */
char *
in_item(item)
char item[];
{
		char *	delim;		/* List of delimiters. */
		char *	start = item;	/* Pointer to start of item. */
	extern	char	Ch;		/* Next input character. */

	switch (Ch) {
	case EOF:	return (NULL);
	case '"':	delim = "\"\n";
			*item++ = Ch; Ch = in_char();
			break;
	default:	delim = "; \t\n";
			break;
	}

	while (index(delim,Ch) == NULL) {	/* While Ch is no delimiter: */
		*item++ = Ch;
		if (item-start >= MAXLEN-1)
			break;
		if ((Ch = in_char()) == EOF)
			break;
	}

	*item = '\0';
	if (Ch == '"')
		Ch = in_char();	/* Skip ending quote. */
	return (Ch == EOF ? NULL : item);
}

/*
 *	IN_CHAR ----------------------------------------------------------------
 *	Read next character from current input stream.
 *	This is the basic input function.
 *	However, for input streams not being the standard input, getline is
 *	called to read the next line in advance, echo it, and put it back
 *	to input. Thus, in_char will still read the line.
 */
char
in_char()
{
	extern	INFILE	In[];
	extern	int	Inlevel;
	extern	char *	getline();
	register char	c;
	register int (*	save_interrupt)();
	extern	int	errno;

	if (Inlevel > 0 && EOL && getline() == NULL) {
		save_interrupt = set_interrupt(INT_OFF);
		pmess("End of input file %s.",FILENAME);
		fclose(STREAM);
		--Inlevel;
		set_interrupt(save_interrupt);
		return (in_char());
	}
	do {
		errno = 0;
		c = BACK > LINE
		  ? *--BACK
		  : getc(STREAM);
	} while (errno == EINTR);
	EOL = (c == '\n' || c == EOF || (Inlevel > 0 && BACK == LINE));
	if (c == EOF) {
		putchar('\n');	/* simulate newline normally typed by user */
		pmess("\"quit\" enforced by end of terminal input.");
	}
	return (c);
}

/*
 *	BACK_CHAR --------------------------------------------------------------
 *	Push back one character to the current input stream.
 */
back_char(c)
char c;		/* Character to be pushed back. */
{
	extern	INFILE	In[];
	extern	int	Inlevel;

	if (BACK < LINE+MAXLEN)
		*BACK++ = c;
	else
		cmd_fatal("back_char","too many push back's");
	EOL = FALSE;
	return;
}

/*
 *	GETLINE -----------------------------------------------------------------
 *	Read ahead a full line from an input stream (which is never level 0).
 *	Then print the line, and put it back (using back_char) for input
 *	by in_char. The sole purpose of this routine is to echo input lines,
 *	read from files, as a whole.
 */
char *
getline()
{
	extern	INFILE	In[];
	extern	int	Inlevel;
		char	line[MAXLEN];
	extern	BOOL	Raw;
	register int	i;
		char	c;

	if (fgets(line,MAXLEN,STREAM) == NULL)
		return (NULL);
	i = strlen(line);
	if (! Raw) {
		fputs(line,stdout);
		if (line[i-1] != '\n')
			/* The input line was truncated by fgets,
				or does otherwise not contain a newline. */
			putchar('\n');
		fflush(stdout);
	}
	if ((c = getc(STREAM)) == EOF) {
		/* On end of file, replace last newline with a space. */
		if (line[i-1] == '\n')
			line[i-1] = ' ';
	}
	else
		ungetc(c,STREAM);
	while (--i >= 0)
		back_char(line[i]);
	return (line);
}

/*
 *	IN_LINE ----------------------------------------------------------------
 *	Read next line from input, i.e. everything up to next newline.
 *	Ch always contains next input character.
 */
char *
in_line(line)
char line[];		/* Place to store line. */
{
	extern	INFILE	In[];
	extern	int	Inlevel;
	register int i = 0;
	extern char Ch;		/* Next input character. */

	while (Ch = in_char(), ! EOL)
		if (i < MAXLEN-1)
			line[i++] = Ch;

	line[i] = '\0';
	return (i == 0 ? NULL : line);
}
/*
 *	ERROR ------------------------------------------------------------------
 *	Print an error message (if not NULL) with a maximum of ten parameters.
 *	Read and print rest of input lines of all input streams,
 *	and skip rest of input streams (except level 0, i.e. stdin).
 *	Executing rest of existing input
 *	would be useless or dangerous after an error occurred.
 */

/* VARARGS 1 */

error(mess,p0,p1,p2,p3,p4,p5,p6,p7,p8,p9)
char *	mess;
char *	p0, *p1, *p2, *p3, *p4, *p5, *p6, *p7, *p8, *p9;
{
	extern	char	Ch;
	extern	INFILE	In[];
	extern	int	Inlevel;
	register int	i;
		char	line[MAXLEN];
	register char	c;
		int (*	save_interrupt)();

	save_interrupt = set_interrupt(INT_OFF);
	if (mess != NULL) {
		printf("%s",Berror);
		printf(mess,p0,p1,p2,p3,p4,p5,p6,p7,p8,p9);
		printf("%s",Eerror);
	}
	for (i = Inlevel; i >= 0; -- i) {
		c = Ch;
		if (! EOL)
			pmess("Input skipped: %c%s",c,
				in_line(line) != NULL ? line : "");
		if (Inlevel > 0) {
			if (ftell(STREAM) <
			    fseek(STREAM,0L,2))  	/* skip rest of file */
				pmess("Rest of file %s skipped.",FILENAME);
			Ch = in_char();    	/* includes message */
		}
	}
	fflush(stdout);
	set_interrupt(save_interrupt);
	return;
}

/*
 *      CMD_SHELL --------------------------------------------------------------
 *	Read rest of input line and pass this to a shell as a command.
 *	On entry, Ch contains '!'.
 */
cmd_shell()
{
		int	status, pid, result;
		char	cmd[MAXLEN];
		char	*shell;
	extern	char	Home[];
	extern	char	Ch;
	register int (*	istat)();
	extern	char	*getenv();

	if (in_line(cmd) == NULL)
		return;
	istat = set_interrupt(INT_OFF);
	if ((pid = fork()) == 0) {
		set_interrupt(INT_DFL);
		chdir(Home);
#ifdef	VERSION6
		execl("/unix7/bin/sh","sh","-c",cmd,0);
#else
		if ((shell = getenv("SHELL")) == NULL)
			shell = "/bin/sh";
		execl(shell,"sh","-c",cmd,0);
#endif	VERSION6
		_exit(127);
	}
	while ((result = wait(&status)) != pid
		&& result != -1)	;
	set_interrupt(istat);
	return;
}

/*
 *	ENDOFLINE --------------------------------------------------------------
 *	Returns TRUE if last character read with in_char was the last in a line.
 */
BOOL
endofline()
{
	extern	INFILE	In[];
	extern	int	Inlevel;

	return (EOL);
}
