#include <stdio.h>
#include <signal.h>
#include <setjmp.h>
#include "cmd.h"
#include "def.h"
#define INITIAL		0
#define IGNORE		1
#define NORMAL		2

/*
 *	SET_INTERRUPT ----------------------------------------------------------
 *	Define in which way interrupts should be handled:
 *	- INT_ON	An interrupt will stop the execution of the current
 *			command, and force a return to the current command level.
 *	- INT_OFF	Interrupts will be ignored. To be used by command
 *			routines that should not be interrupted.
 *	- INT_DFL	Interrupts will stop the program.
 *	- otherwise	Parameter is assumed to be a function address, and is
 *			directly passed to "signal".
 *	"Command" always sets the interrupt on, before reading and executing
 *	the next command.
 *	However, if the interrupt is off when set_interrupt is called for the
 *	first time, it will stay off forever, regardless the values of the
 *	"func" parameter in future "set_interrupt" calls.
 */
int (*
set_interrupt(func))()
int (*	func)();
{
	extern	int	cmd_interrupt();
		int (*	old_int)();
	static	int	IntMode = INITIAL;

	old_int = signal(SIGINT,SIG_IGN);
	if (IntMode == INITIAL)
		IntMode = old_int == SIG_IGN ? IGNORE : NORMAL;
	if (IntMode == NORMAL)
		signal(SIGINT,func);
	return (old_int);
}

/*
 *	CMD_INTERRUPT ----------------------------------------------------------
 *	Handle a caught interrupt: print message,
 *	execute recover function and return to current command level.
 */
cmd_interrupt()
{
	extern	jmp_buf	CmdEnv;

	signal(SIGINT,SIG_IGN);
	putchar('\n');
	error("Command interrupted.");
	recover();
	longjmp(CmdEnv,1);
}
