/*
    vmehb - VME bus driver of PCI/VME bus adaptor Model 617 of Bit3.
    ----------------------------------------------------------------
    vmehb.c - main body of the driver, smaall dependence of System
   	      flags because of changing the typing of parameters
	      in 2.1 series. 2.1 compatible. Demands interrupt-proof
	      semaphore.
    Copyright (C) May 1997  Natalia Kruszynska <natalia@nikhef.nl>
	Thanks for contribution to:
		Andries Brouwer <aeb@cwi.nl>
		Marcin Dalecki  <dalecki@sub944.sub.uni-goettingen.de>
		David Grothe	<dave@gcom.com>
		Henk Peek	<henkp@nikhef.nl>
		Ruud van Wijk   <ruud@nikhef.nl>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
 * The following  prefixes are consequently used in the code:
 *	vme_		- configuration data of a vme space
 *	bit3_		- device  structure members
 *	uio_		- request structure members (UNIX hickup)
 *	mmap_		- mmap structure members (mmap call )
 *	vmehb_		- big routines of the driver 
 *	Va_		- base auxiliary routines (vmehb_aux.c)
 *	Vm_		- semaphoring & setup inlines from vme_conf.h
 *	Bit3_		- hardware access inline primitives
 *	set_		- tracing functions, mostly inactive
 */

/* ================================================================== */
#include	"vmehb_conf.h"

/* ================================================================== */
/* FILE OPERATIONS  start */
/* -----------------------------------*/
#include	"vmehb_aux.c"
#ifdef	LIN2_0
/* PATCH1.0.0
 * lseek   is  implemented because of 1Gb limit of file->f_pos
 * It works only in absolute way - this is the feature ;-)))))
 */
static int vmehb_lseek(struct inode * inode, struct file * file, off_t offset, int orig)
{
	/* we have to put it to deal with 32 bits offset of vme32	*/
	file->f_pos = offset;
	return(0);
}
#endif
#ifdef	LIN2_1_27
/* PATCH1.0.0
 * lseek is not implemented   because of  32 bits  file->f_pos
 * It works only in absolute way - this is the feature ;-)))))
 */
static long vmehb_read(struct inode *node,struct file *file,char *buf
					, unsigned long count)
#endif
#ifdef	LIN2_0
static int vmehb_read(struct inode *node,struct file *file,char *buf
					, int count)
#endif
{
	int 	err;
	if(bit3_VME_on == 0)
		return(-ENODEV);
	down(&bit3_sem);	 /* SEM */
	err = Va_check_access((u32)(file->f_pos),buf,count,VERIFY_WRITE,vme_minor);
	if(err)
	{
		up(&bit3_sem);	 /* SEM */
		return(err);
	}
	/* this flag is set compile-time or later by ioctl	*/
	/* excluding DMA and invoking the delayed PIO access	*/
	/* to check the VME configuration for VME bus errors	*/
	if(bit3_pio_delay)
		goto pio_access;
	/* this flag is set compile-time or later by ioctl	*/
	/* excluding DMA and invoking the PIO access for slow	*/
	/* control access with mmap allowed			*/
	if(bit3_slow_control)
		goto pio_access;
	/* this is the DAQ trigger DMA/PIO distribution 	*/
	if(count > vme_pio) /**/
		goto dma_access;
	else
		goto pio_access;
	/**/
dma_access:
	if((err = Va_dma_map(file->f_pos,(char *)buf,count,VME_READ)))
	{
		up(&bit3_sem);	 /* SEM */
		return(err);
	}
	set_A(0xFF001133);		/* TRACING A */
	set_A(current->pid);		/* TRACING A */
	set_A(more_rdma);		/* TRACING A */
	count = Va_dma_access((u32)(file->f_pos),count,(char *)buf,VME_READ);
	up(&bit3_sem);		/* SEM */
	return(count);
pio_access:
	if((err = Va_pio_map((u32)(file->f_pos),count)) < 0)
	{
		up(&bit3_sem);	 /* SEM */
		return(err);
	}
	set_A(0xFF002233);		/* TRACING A */
	set_A(current->pid);		/* TRACING A */
	count = Va_pio_access((u32)(file->f_pos),count,buf,VME_READ);
	up(&bit3_sem);	 /* SEM */
	return(count);
} /* end vmehb_read() */
/* -----------------------------------*/
#ifdef LIN2_1_27
static long vmehb_write(struct inode *node,struct file *file,const char *buf
					,unsigned long count)
#endif
#ifdef LIN2_0
static int vmehb_write(struct inode *node,struct file *file,const char *buf
			,int count)
#endif
{
	int 	err;
	if(bit3_VME_on == 0)
		return(-ENODEV);
	down(&bit3_sem);	 /* SEM */
	err = Va_check_access((u32)(file->f_pos),(char *)buf,count,VERIFY_READ,vme_minor);
	if(err)
	{
		up(&bit3_sem); 	/* SEM */
		return(err);
	}
	/* this flag is set compile-time or later by ioctl	*/
	/* excluding DMA and invoking the delayed PIO access	*/
	/* to check the VME configuration for VME bus errors	*/
	if(bit3_pio_delay)
		goto pio_access;
	/* this flag is set compile-time or later by ioctl	*/
	/* excluding DMA and invoking the PIO access for slow	*/
	/* control access with mmap allowed			*/
	if(bit3_slow_control)
		goto pio_access;
	/* this is the DAQ trigger DMA/PIO distribution 	*/
	if(count > vme_pio) /**/
		goto dma_access;
	else
		goto pio_access;
	/**/
dma_access:
	if((err = Va_dma_map(file->f_pos,(char *)buf,count,VME_WRITE)) < 0)
	{
		up(&bit3_sem);	 /* SEM */
		return(err);
	}
	set_A(0xFF001144);		/* TRACING A */
	set_A(current->pid);		/* TRACING A */
	set_A(more_wdma);		/* TRACING A */
	count = Va_dma_access((u32)(file->f_pos),count,(char *)buf,VME_WRITE);
	up(&bit3_sem);		 /* SEM */
	return(count);
pio_access:
	if((err = Va_pio_map((u32)(file->f_pos),count)) < 0)
	{
		up(&bit3_sem);	 /* SEM */
		return(err);
	}
	set_A(0xFF002244);		/* TRACING A */
	set_A(current->pid);		/* TRACING A */
	count = Va_pio_access((u32)(file->f_pos),count,(char *)buf,VME_WRITE);
	up(&bit3_sem);	 /* SEM */
	return(count);
} /* end vmehb_write() */
static int vmehb_mmap(struct inode *node,struct file *file
				,struct vm_area_struct *vma)
{
	int err,size,PCIoff;
	/* MMAP implementation:
	** I do not allow programs to mmap when we are running
	** DMA access, because we can not prevent the Adaptor
	** Cable to be accessed by the mapping tasks during DMA.
	** I assume that the Manager decides on one mode at
        ** a time. The community of Linux hackers is challenged 
	** to find the solution allowing mapped pages to "hang on" 
	** waiting  for DMA done instead of waiting for 
	** "swap back" ;-)))  (I am not a hacker, I am a real time 
	** programmer.... I don't d(c)are )
	**      (C)		May 97 <natalia@nikhef.nl>
	*/
	/*
	** current notes: limitation to slow or check modes removed
	** for the test time:
	if(bit3_slow_control == 0 && bit3_pio_delay == 0)
		return(-ENODEV);
	*/
	down(&bit3_sem); 	/* SEM */
	err = Va_check_access((u32)(vma->vm_offset),(char *)0,0x1000,VERIFY_READ,vme_minor);
	if(err)
	{
		up(&bit3_sem); 	/* SEM */
		return(err);
	}
	size = vma_get_end(vma) - vma_get_start(vma);
	if((err = Va_map_map((u32)(vma->vm_offset),size)) < 0)
	{
		up(&bit3_sem); 	/* SEM */
		return(err);
	}
	PCIoff	= err * 0x1000;
	if((remap_page_range(vma_get_start(vma)
			   ,PCI_conf._base3 + PCIoff
			   ,size
			   ,vma_get_page_prot(vma))))
	{
		up(&bit3_sem); 	/* SEM */
		return(-ENOMEM);
	}
	/* if head				*/
	if(bit3_first == (struct vme_frag *)0)
	{
		bit3_first = (struct vme_frag *)kmalloc(sizeof(struct vme_frag)
					,GFP_KERNEL);
		BIT3_TRACE(("vmehb: set head for mmap to %x\n",bit3_first));
		bit3_next = bit3_first;
		if(bit3_first == (struct vme_frag *)0)
		{
			up(&bit3_sem); 	/* SEM */
			printk("Cannot alloc list head for map\n");
			return(-EIO);
		}
	}
	else
	{
		bit3_next->next = (struct vme_frag *)kmalloc(sizeof(struct vme_frag)
					,GFP_KERNEL);
		BIT3_TRACE(("vmehb: set next for mmap to %x\n",bit3_next->next));
		if(bit3_next->next == (struct vme_frag *)0)
		{
			up(&bit3_sem); 	/* SEM */
			printk("Cannot alloc list slot for map\n");
			return(-EIO);
		}
		bit3_next	= bit3_next->next;
	}
	bit3_next->pid	= current->pid;
	bit3_next->vadr = vma->vm_offset;
	bit3_next->from	= uio_n_map;
	bit3_next->pages= uio_n_len;
	bit3_next->next	= (struct vme_frag *)0;
	Vm_list_maps();
	vma_set_inode(vma,node);
	inode_inc_count(node);
	up(&bit3_sem); 				/* SEM */
	return(0);
} /* end vmehb_mmap() */
static int vmehb_ioctl(struct inode *node,struct file *file,
			unsigned int cmd, unsigned long arg)
{
	int size,err;
	long argval;
	vme_dev_p = (struct vme_dev	*)&(vme_config[vme_minor]);
	size = _IOC_SIZE(cmd);
	switch(cmd) {
	case	VME_GET_SPACE:  /* IOCTL b 0 */
		err = verify_area(VERIFY_WRITE,(void *)arg,size);
		if(err)
			return(err);
		Vm_copy_to_user((char *)arg,(char *)vme_dev_p,size);
		return(0);
	case	VME_SET_SPACE:	/* IOCTL b 1 */
		if(!suser())
			return(-EPERM);
		err = verify_area(VERIFY_READ ,(void *)arg,size);
		if(err)
			return(err);
		down(&bit3_sem);		/* SEM */
		Vm_copy_from_user((char *)vme_dev_p,(char *)arg,size);
		up(&bit3_sem);			/* SEM */
		return(0);
	case	VME_CHECK_ACCESS:	/* IOCTL b 2 */
		err = verify_area(VERIFY_READ ,(void *)arg,size);
		if(err)
			return(err);
		Vm_get_uval(argval,(long *)arg); /**/
		if(bit3_pio_delay == argval)
		{
			Vm_report_driver();
			return(0);
		}
		if(argval < 0 || argval > 400)
			return(-EINVAL);
		if(!suser())
			return(-EPERM);
		down(&bit3_sem);		/* SEM */
		bit3_pio_delay = argval;
		up(&bit3_sem);			/* SEM */
		Vm_report_driver();
		return(0);
	case	VME_SLOW_CONTROL:	/* IOCTL b 3 */
		err = verify_area(VERIFY_READ ,(void *)arg,size);
		if(err)
			return(err);
		Vm_get_uval(argval,(long *)arg); /**/
		if(bit3_slow_control == argval)
		{
			Vm_report_driver();
			return(0);
		}
		if(argval < 0 || argval > 400)
			return(-EINVAL);
		if(!suser())
			return(-EPERM);
		down(&bit3_sem);		/* SEM */
		bit3_slow_control = argval;
		up(&bit3_sem);			/* SEM */
		Vm_report_driver();
		return(0);
	case	VME_DMA_CONTROL:	/* IOCTL b 4 */
		err = verify_area(VERIFY_READ ,(void *)arg,size);
		if(err)
			return(err);
		Vm_get_uval(argval,(long *)arg); /**/
		if(bit3_irq_used == argval)
		{
			Vm_report_driver();
			return(0);
		}
		if(argval < 0 || argval > 400)
			return(-EINVAL);
		if(!suser())
			return(-EPERM);
		down(&bit3_sem);		/* SEM */
		bit3_irq_used = argval;
		up(&bit3_sem);			/* SEM */
		Vm_report_driver();
		return(0);
	case	VME_SET_AM:	/* IOCTL b 5 */
		if(!suser())
			return(-EPERM);
		err = verify_area(VERIFY_READ ,(void *)arg,size);
		if(err)
			return(err);
		Vm_get_uval(argval,(long *)arg); /**/
		down(&bit3_sem);		/* SEM */
		vme_am = argval;
		up(&bit3_sem);			/* SEM */
		printk("vmebh: dev %s , minor %d am changed to x%x\n"
			,vme_name
			,vme_minor
			,vme_am
			);
		return(0);
	case	VME_SET_DMAMOD:	/* IOCTL b 6 */
		if(!suser())
			return(-EPERM);
		err = verify_area(VERIFY_READ ,(void *)arg,size);
		if(err)
			return(err);
		Vm_get_uval(argval,(long *)arg); /**/
		if(    argval != 0 
		   &&  argval != RC2_DMA_BLK_SEL
	           &&  argval != RC2_DMA_PAUSE)
			return(-EINVAL);
		down(&bit3_sem);		/* SEM */
		vme_dmamod = argval;
		up(&bit3_sem);			/* SEM */
		printk("vmehb: dev %s , minor %d dmamod changed to %s\n"
			,vme_name
			,vme_minor
			,argval == RC2_DMA_PAUSE   ? "pause" :
			 argval == RC2_DMA_BLK_SEL ? "block" : "non block"
			);
		return(0);
	case	VME_CRATE_OFF:	/* IOCTL b  10 */
		if(bit3_VME_on == 0)
		{
			printk("vmehb: Vme crate already off\n");
			return (0);
		}
		if(!suser())
			return(-EPERM);
		printk("vmehb: Vme crate going   off\n");
		down(&bit3_sem);		/* SEM */
		bit3_VME_on = 0;
		up(&bit3_sem);			/* SEM */
		return(0);
	case	VME_CRATE_ON: /* IOCTL b  11  */
		if(bit3_VME_on == 1)
		{
			printk("vmehb: Vme crate already on\n");
			return 0;
		}
		if(!suser())
			return(-EPERM);
		down(&bit3_sem);		/* SEM */
		printk("vmehb: Vme crate going   on \n");
		if(Bit3_init(bit3_base0)) 
		{
			bit3_VME_on = 0;
			printk("ioctl can not initialize VME crate\n");
			return(-ENODEV);
		}
		bit3_VME_on = 1;
		up(&bit3_sem);			/* SEM */
		return(0);
	case	VME_CLEAR_MAPS:		/* IOCTL b 61 */
		if(!suser())
			return(-EPERM);
		printk("vmehb: suser unmapping all mappings!!!\n");
		Vm_list_maps();
		down(&bit3_sem);		/* SEM */
		Vm_clear_maps(0);
		up(&bit3_sem);			/* SEM */
		return(0);
	case	VME_TRACE_ZOMBIES:	/* IOCTL b 62 */
		if(!suser())
			return(-EPERM);
		if(bit3_wakeups)
		{
			printk("Try to wake a sleeping beauty\n");
zombie2:
			set_A(0xFF00FF66);		/* TRACING A */
			set_A(bit3_wait_struct->task->pid);		/* TRACING A */
			if((bit3_wait_struct->task->pid) <= 0)
			{
				printk("No sleeping beauties %x  seen (wake %d)\n"
					,(int)bit3_wait_struct->task
					,bit3_wakeups);
				if(bit3_wakeups) --bit3_wakeups;
				print_trace();
			}
			else
			{
				set_A(bit3_wait_struct->task->pid);		/* TRACING A */
				printk("Kissing a beauty task %x :%d (%x)\n"
					,(int)bit3_wait_struct->task
					,bit3_wait_struct->task->pid
					,bit3_wait_struct->task->pid
					);
				print_trace();
				bit3_wakeups--;
				wake_up(bit3_wait_q);	/* wakeups */
			}
		}
		else
		{
			printk("bit3_wakeups == 0 , try anyway\n");
			goto zombie2;
		}
		return(0);
	case	VME_TRACE_INFO:		/* IOCTL b 63 */
		print_trace();
		Vm_list_maps();
		return(0);
	default:
		return -EINVAL;
	}
}
/* -----------------------------------*/
static int vmehb_open(struct inode *node,struct file *file)
{
	MOD_INC_USE_COUNT;
	/**/
	return	0;
}
/* -----------------------------------*/
static void vmehb_release(struct inode *node,struct file *file) {
	MOD_DEC_USE_COUNT;
	/**/
	BIT3_TRACE(("vmehb: release unmaps process %d\n",current->pid));
	down(&bit3_sem);			/* SEM */
	Vm_clear_maps(current->pid);
	Vm_list_maps ();
	up(&bit3_sem);				/* SEM */
	/**/
	BIT3_TRACE(("vmehb decrements node\n"));
}
/* -----------------------------------*/
static struct file_operations vmehb_ops = {
#ifdef LIN2_0
	vmehb_lseek,		
#endif
#ifdef	LIN2_1_27
	NULL,			/* use system's & check by r/w */
#endif
	vmehb_read,
	vmehb_write,
	NULL,
	NULL,
	vmehb_ioctl,		/* ioctl	*/
	vmehb_mmap,		/* mmap		*/
	vmehb_open,
	vmehb_release,
	NULL,
	NULL,
	NULL,
	NULL,
};
/* ================================================================== */
/* FILE OPERATIONS  end   */
/*
 * INTERRUPT ------------------------------------
 * here is interrupt handler, attached in init_mod
 * we try to make a good service routine out of him
 * we use goto as recommended by gnu ;-)))))))))))
 */
/**/
static void vmehb_irqhandler(int bit3_irq,void *dev_id,struct pt_regs *regs)
{
	int v;
	int command;
	int count;
/* old_pending: */
	if((bit3_error & 0x10))
	{
		printk("DMA pending from %d(x%x)\n"
			,bit3_dma_pid
			,bit3_dma_pid);
		bit3_error &= ~(0x10);
	}
	if(bit3_error)
	{
		printk("VME device error %x,user not notified\n",bit3_error);
		bit3_error = 0;
	}
	count = 0;
/* pending: */
	/* alien ? return ! */
	if(Bit3_irq_alien(bit3_base0)) 
		return;
	count++;
	command = 0;
/* errors: */
	v = Bit3_r_get(bit3_base0,LOC_STATUS);
	if(v & LSR_CERROR_MASK)
	{
		command |= LC1_CLR_ERROR;
		Bit3_r_set(bit3_base0,LOC_CMD1,command);
		if(v & LSR_TIMEOUT_ERR) /**/
		{
			Bit3_r_get(bit3_base0,REM_PAGE_LO);
			Bit3_r_get(bit3_base0,REM_PAGE_HI);
		}
		bit3_error	|= v;
		if(v & LSR_NO_CONNECT) /**/
		{
			set_A(0xFF009999);	/* TRACING A */
			set_A(v);		/* TRACING A */
			set_A(bit3_error);	/* TRACING A */
			printk("VME crate is OFF!!\n");
			bit3_VME_on = 0;
			bit3_error  |= LSR_NO_CONNECT;
			Bit3_clear_status(bit3_base0);
			print_trace();
			return;
		}
		else
		{
			bit3_VME_on = 1;
			set_A(0xFF000088);	/* TRACING A */
			set_A(v);		/* TRACING A */
			set_A(bit3_error);	/* TRACING A */
		}
		if(bit3_dma_busy)
		{
			printk("VME %x  pid %d\n",bit3_error, bit3_dma_pid);
		}
		return;
	} /* end of errors */
/* pr_interrupt: */
	v = Bit3_r_get(bit3_base0,LOC_STATUS);
	if(v & LSR_PR_STATUS)
	{
		command |= LC1_CLR_PR_INT;
		Bit3_r_set(bit3_base0,LOC_CMD1,command);
		return;
	} /* end of pr_interrupt */
/* dma_done: */
	if(count > 1)
		printk("More than one turn on pending in IRQ\n");
	v = Bit3_r_get(bit3_base0,LDMA_CMD);
	set_A(0xFF000077);	/* TRACING A */
	set_A(v);		/* TRACING A */
	set_A(more_irq);	/* TRACING A */
	set_A(bit3_wakeups);	/* TRACING A */
	if(v & LDC_DMA_DONE)
	{
		/* Bit3_clear_status(bit3_base0); /**/
		v &= ~LDC_DMA_DONE;
		Bit3_r_set(bit3_base0,LDMA_CMD,v);
		bit3_error |= (1 << 4);
		count = 0;
		while(bit3_wakeups)
		{
			VMEHB_TRACE(("Wake up IRQ\n"));
			set_A(0xFF000066);		/* TRACING A */
			set_A((int)bit3_wait_struct->task->pid);		/* TRACING A */
			if(((int)(bit3_wait_struct->task->pid)) <= 0)
			{
				udelay(100);
				printk("IRQ - no sleeper yet, delay 100 usec\n");
				if(count++ > 10)
				{
					printk("IRQ lost task to wake up\n");
					return;
				}
				continue;
			}
			else
			{
				set_A(bit3_wait_struct->task->pid);		/* TRACING A */
				bit3_wakeups--;
				wake_up(bit3_wait_q);	/* wakeups */
			}
		}
		/* try here later return DEBUG */
		return;
	}
/* pt_interrupt: */
	printk("Unimplemented interrupts beyond DMA DONE\n");
	v = Bit3_r_get(bit3_base0,REM_STATUS);
	/* on Remote Status Register */
/* vme_backplane_interrupt: */
	/* on Local Interrupt Register */
/* spurious: */
	BIT3_TRACE(("Something not cleared yet ??? \n"));
	Bit3_clear_status(bit3_base0);
	set_A(0xFF000055);		/* TRACING A */
	return;
	
}
/* ================================================================== */
/* PCI bus init */
#include	"pci_dev_probe.c"
/* WRAPPER	start*/
int init_module(void)
{
	int	addr[64];
	if(!pcibios_present())
	{
		BIT3_TRACE(("vmehb: can not find pci bios\n"));
		return -ENODEV;
	}
	if(pci_dev_config(PCI_VENDOR_ID_BIT3,
	            PCI_DEVICE_ID_BIT3_VME,&PCI_conf,"vmehb"))
	{
		pci_dev_info(PCI_conf,"vmehb");
		bit3_irq = PCI_conf._int_line;
		bit3_base0 = PCI_conf._base0 & 0xfffffffc;
		/* PATCH1.0.0
	         * in the dense packed PCI  configurations lack of  mask
		 * over the I/O port address can cause unjust failure of
		 * check_region interpreting  type bits as start address 
		 */
		if((addr[0] = check_region(PCI_conf._base0 & 0xfffffffc,0x20))) 
		{
			BIT3_TRACE((" Region 0 on %x does not check \n" 
				,PCI_conf._base0));
		}
		if( Bit3_init(bit3_base0))
			bit3_VME_on = 0;
		else
			bit3_VME_on = 1;
		Bit3_init(bit3_base0);	/* DEBUG no check on VME powered */
		/* PATCH1.0.0
	         * in the dense packed PCI  configurations lack of  mask
		 * over the I/O port address can cause unjust failure of
		 * check_region interpreting  type bits as start address 
		 */
		if((addr[1]=check_region(PCI_conf._base1 & 0xfffffffc,0x10000)))
		{
			BIT3_TRACE((" Region 1 on %x does not check\n"
				,PCI_conf._base1));
		}
		if((addr[2]=check_region(PCI_conf._base2,0x10000)))
		{
			BIT3_TRACE((" Region 2 on %x does not check\n"
				,PCI_conf._base2));
		}
		if((addr[3]=check_region(PCI_conf._base3,0x2000000)))
		{
			BIT3_TRACE((" Region 3 %x does not check\n"
				,PCI_conf._base3));
		}
		if(addr[0] || addr[1] || addr[ 2] || addr[ 3] )
		{
			BIT3_TRACE((" check failure: %x %x %x %x \n"
				,addr[ 0],addr[1],addr[ 2],addr[ 3]));
			return(-ENODEV);
		}
		BIT3_TRACE(("Regions checked, SA_INTERRUPT for a try\n"));	
		if(request_irq(bit3_irq,vmehb_irqhandler,SA_INTERRUPT,"vmehb",NULL))
		{
			BIT3_TRACE(("vmehb: interrupt registration fault on level %d\n"
				,bit3_irq));
			return(-ENODEV);
		}
		/* we service normal & error, PT by ioctl only */
		Bit3_set_intr(bit3_base0,1,1,0);
		/* worked witout this enable in 2.0.0, here not?? */
		/* enable_irq(bit3_irq); /**/
		request_region(bit3_base0,0x20,"bit3_ior_0");
		BIT3_TRACE(("Remapping regions 2 & 3\n"));	
		bit3_base2 = (int)vremap(PCI_conf._base2,0x10000	);
		bit3_base3 = (int)vremap(PCI_conf._base3,0x2000000	);
		BIT3_TRACE(("vremapped:  %8x %8x \n"
			,bit3_base2,bit3_base3));
		if(bit3_base2 == 0 || bit3_base3 == 0 )
		{
			BIT3_TRACE(("vmehb can not map into kernel!!\n"));
			printk("vmehb can not map into kernel!!\n");
			free_irq(bit3_irq,NULL);
			release_region(bit3_base0,0x20	);
			return(-ENOMEM);	/* init phase	*/
		}
	}
	/* 
	 *  funny ways with the dynamic assignment of minor nrs 
	 *  when BIG_MINOR =0 
	 */
	if((bit3_major= register_chrdev(BIT3_MAJOR,"vmehb",&vmehb_ops   )) < 0) 
	{
		BIT3_TRACE(("vmehb no major %d (fault %d) \n",BIT3_MAJOR,bit3_major));
		Bit3_set_intr(bit3_base0,0,0,0);
		free_irq(bit3_irq,NULL);
		release_region(bit3_base0,0x20	);
		return -EIO;
	}
	if (BIT3_MAJOR > 0)
		bit3_major = BIT3_MAJOR;
	BIT3_TRACE(("LOAD vmehb  as MODULE %d ( asked %d)  irq %d\n"
		,bit3_major,BIT3_MAJOR,bit3_irq));
	/* set up the device - see in vmehb_conf.h	*/
	Vm_set_me_up();
	return 0;
}
void cleanup_module(void)
{
	BIT3_TRACE(("UNLOAD vmehb %d ( %d) free intr %d\n"
			,bit3_major,BIT3_MAJOR,bit3_irq));
	/* disable all interrupts			*/
	Bit3_set_intr(bit3_base0,0,0,0);
	free_irq(bit3_irq,NULL);
	vfree((void *)bit3_base3);
	vfree((void *)bit3_base2);
	release_region(bit3_base0,0x20	);
	if(bit3_major > 0)
		unregister_chrdev(bit3_major,"vmehb");
	else
		unregister_chrdev(BIT3_MAJOR,"vmehb");
}
/* WRAPPER	end  	*/
