
/*
    vmehb - VME bus driver of PCI/VME bus adaptor Model 617 of Bit3.
    ----------------------------------------------------------------
    vmehb_aux.c - intrinsic part of the driver, to be included where
		it is already included in vmehb.c. Separated to put
		apart medium level routines.
    Copyright (C) May 1997  Natalia Kruszynska <natalia@nikhef.nl>
	Thanks for contribution to:
		Andries Brouwer <aeb@cwi.nl>
		Marcin Dalecki  <dalecki@sub944.sub.uni-goettingen.de>
		David Grothe	<dave@gcom.com>
		Henk Peek	<henkp@nikhef.nl>
		Ruud van Wijk   <ruud@nikhef.nl>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
 * The following  prefixes are consequently used in the code:
 *	vme_		- configuration data of a vme space
 *	bit3_		- device  structure members
 *	uio_		- request structure members (UNIX hickup)
 *	vmehb		- big routines of the driver 
 *      Va_		- base auxiliary routines
 *	Vm_		- semaphoring & setup inlines from vme_conf.h
 *	Bit3_		- hardware access inline primitives
 */
	/* 
         *   bit3_irq_used flag is used to switch the mode of DMA service
	 *   bit3_irq_used is 1 for IRQ service of DMA, negative for POLL
	 *   Absolute values of negative  bit3_irq_used  mean poll mode 
	 *   with :
	 *    < 1000	udelay  of (-bit3_irq_used) usec used on polling
	 *    > 1000    fancy tricks used for testing the interrupts
         */

static int Va_dma_count(u32 pos)
{
	u32 val;
	u32 cc;
	int count;
	/* calculate VME pos from registers here	*/
	val = Bit3_err_addr(bit3_base0) ;
	cc = val - pos;	
	if (cc & 0x80000000)
	{
		cc = 0;
		val = 0;
	}
	BIT3_TRACE(("cc %x val %x pos %x\n",cc,val,pos));
	count = cc;
	if(bit3_error & LSR_NO_CONNECT){ 
           printk("DMA: switch VME crate ON!!!\n" );return(0);}
	if(bit3_error & LSR_PARITY_ERR)
	   printk( "DMA: parity error VME range [%x,%x]\n" ,(int)pos,(int)val);
	if(bit3_error & LSR_REMBUS_ERR)
	   printk( "DMA: BERR on VME range [%x,%x]\n" ,(int)pos,(int)val);
	if(bit3_error & LSR_TIMEOUT_ERR)
	   printk( "DMA: timeout on VME range [%x,%x]\n" ,(int)pos,(int)val);
	if(bit3_error & LSR_LRC_ERR)
	   printk( "DMA: LRC on VME [%x,%x]\n" ,(int)pos,(int)val);
	if((bit3_error & LSR_ERROR_MASK) == 0)
	   BIT3_TRACE(("DMA: access on VME [%x,%x]\n",(int)pos,(int)val));
	bit3_error = 0; 
	return(count);
} /* end Va_dma_count(u32 pos) /**/
static int Va_check_access(u32 pos, char *buf,int count,
			int verify,int minor)
{
	/* if buf == 0, pos = vma->vm_offset , usable in mmap	*/
	int err;
	char	*fun;
	/* frozen mapping ahead	DEBUG */
	bit3_free_ahead = 0;
	vme_dev_p = (struct vme_dev	*)&(vme_config[minor]);
	uio_p	  = (struct io_ahead	*)&(bit3_dev_p->aheads[bit3_free_ahead]);
	bit3_free_ahead 	= -1;
	uio_align 		= 0;
	uio_pid	  		= current->pid;

	/* check the address range			*/
	if((u32)(pos) > (u32)vme_maxad)
		 return(-EFAULT); 

	/* for mmap - align to the page size & return	*/
	if(buf == (char *)0)
	{
		if((pos) & 0xfff)
			return(-EINVAL);
		return(0);
	}

	/* for read/write - align to the data size & verify & return	*/
	/* only aligned counts , round to aligned 			*/
	if((pos) & (vme_mod - 1))
	{
		BIT3_TRACE (("pos %x\n",pos));
		return(-EFAULT);
	}
	while(count & (vme_mod - 1))
	{
		count++;
		uio_align++;
	};
	fun = (verify == VERIFY_READ  ? "write" : 
              (verify == VERIFY_WRITE ? "read"  : "panic"));
	if((err = verify_area(verify, buf,count)))
	{
		BIT3_TRACE(("vmehb: not verified %s area %x\n",fun,(int)buf));
		return err;
	}
	fun = (verify == VERIFY_READ  ? "write" : 
              (verify == VERIFY_WRITE ? "read"  : "panic"));
	BIT3_TRACE(("For %d %s\n",uio_pid ,fun));
	return(0);
} /* end Va_check_access */
static int Va_dma_map(u32 pos,char *buf,int count, int rw) 
{
	int val,i,err;
	uio_n_len	= count >> 12;
	if(count & 0xfff)
		uio_n_len += 1;
	uio_n_map	= Bit3_map_alloc(bit3_base2,MR_DMA_PCI,uio_n_len); /* DEBUG */
	uio_addr	= Vm_dma_mem_alloc(count);	/**/
	set_B(0xEE000011);		/* TRACING B */
	set_B(current->pid);		/* TRACING B */
	set_B(uio_addr);		/* TRACING B */
	set_B(count);			/* TRACING B */
	if(uio_addr == 0)
	{
		printk("vmehb: Va_dma_map - just  no memory for dma, retry\n");
		print_trace();
		udelay(100);
		uio_addr	= Vm_dma_mem_alloc(count);	/**/
		if(uio_addr == 0)
		{
			printk("vmehb: Va_dma_map - again no memory for dma, retry\n");
			udelay(10);
		        uio_addr	= Vm_dma_mem_alloc(count);	/**/
			if(uio_addr == 0)
			{
				printk("vmehb: Va_dma_map - still no memory for dma\n");
				return(-ENOMEM);
			}
		}
	}
	if(uio_addr & 0xff000000)
	{
		printk("vmehb: warning, dma address high %x\n",uio_addr);
	}
	uio_vadr	= pos;
	uio_dam		= vme_am;
	BIT3_TRACE(("DMA allocated on: %x map %x len %x align %x\n"
			,uio_addr,uio_n_map,uio_n_len,uio_align));
	if(bit3_dma_busy)
	{
		printk("dma should never be busy here\n");
	}
	uio_rw		= rw;		/* for write, for read 	*/
	if(uio_rw == VME_WRITE)
	{
		BIT3_TRACE(("From user\n"));
		Vm_copy_from_user((char *)uio_addr,buf,count - uio_align);	/**/
	}
	/* map the registers */
	val = DMAREG(uio_addr ,vme_swap,0);
	for(i = 0;i < uio_n_len;i++)
	{
		err=Bit3_p_set(bit3_base2,MR_DMA_PCI,uio_n_map + i,val);
		val += 0x1000;
	}
	/* set the CSR's	*/
	val = BIT3DMA_COMMAND(vme_mod,vme_fu,uio_rw,0);
	/* the next call should have one more param = uio_n_map */
	Bit3_ldma(bit3_base0,uio_n_map,uio_addr,count,val);
	return(0);
} /* end Va_dma_map() */
static int Va_dma_access(u32 pos,int count,char *buf,int rw)
{
	int  cnt;
	unsigned long flags;
	bit3_dma_busy = 1;
	cnt = count;
	BIT3_TRACE(("vmehb_dma_access pos %x\n",pos));
	Bit3_rdma(bit3_base0,uio_vadr,count);
	vme_dmamod |=  RC2_CINT_DISABLE;
	/* start DMA	(0 = poll, 1 = interrupt)		*/
	Bit3_lreg(bit3_base0,1,vme_dmamod,vme_am); /* 	*/
	DUMP_REGISTERS(bit3_base0); /* writes on VME 	*/
	bit3_dma_pid = current->pid;
	if(bit3_irq_used > 0) 	/* IRQ mode	*/
	{
		bit3_wakeups++;	
/* START CRITICAL */
		save_flags(flags);		
		cli();			
   		Bit3_dma_start(bit3_base0,bit3_irq_used);
		sleep_on(bit3_wait_q);
		restore_flags(flags);
/* END   CRITICAL */
	}
	else	/* POLL modes */
	{
   		Bit3_dma_start(bit3_base0,bit3_irq_used); 
	}
	if(bit3_dma_pid != current->pid)
	{
		printk("slept as %d woken %d\n",bit3_dma_pid,current->pid);
	}
	count = Va_dma_count(pos);   /* in vmehb_aux.c */
        DUMP_REGISTERS(bit3_base0); /* writes on VME 	*/
/* clean_dma: */
	/* clean the administration of DMA */
	if(cnt == count)
		count -=uio_align;
	if(uio_rw == VME_READ )
	{
		BIT3_TRACE(("To   user\n"));
		Vm_copy_to_user(buf,(char *)uio_addr,count  );	/**/
	}
	if(uio_addr)
	{
		set_B(0xEE000022);		/* TRACING B */
		set_B(current->pid);		/* TRACING B */
		set_B(uio_addr);		/* TRACING B */
		set_B(count);			/* TRACING B */
		Vm_dma_mem_free(uio_addr,count);
	}
	/* Bit3_map_free (bit3_base2,MR_DMA_PCI,uio_n_len); /* DEBUG */
	uio_pid		= 0;
	bit3_free_ahead = 0;
	bit3_dma_busy	= 0;
	bit3_error	= 0;
	return(count);
} /* end vmehb_dma_access() */
static int Va_pio_map(u32 pos,int count)
{
	int	i;
	int cnt,val,err;
	cnt		= count;
	uio_off	= pos & 0xfff;
	uio_n_len	= ((pos + count ) >> 12) -
                          ( pos           >> 12) + 1 ;
	if(bit3_dma_busy)
	{
		/* we come here only to implement readahead  NIY	*/
		/* down(&bit3_sem)	.... e.t.c, e.t.c.... 		*/
	}
	/* PIO para maps onto the first  2 reserved mapping registers 	*/
	/* the routine will return 0 , see  bit3_regs.h			*/
	uio_n_map	= Bit3_map_alloc(bit3_base2,MR_PCI_VME,uio_n_len); /* DEBUG */
	val = BIT3REG((u32)pos,vme_am,vme_fu,vme_swap,0); /**/
	for(i = 0;i < uio_n_len;i++)
	{
		err=Bit3_p_set(bit3_base2,MR_PCI_VME,uio_n_map + i,val);
		val += 0x1000;
	}
	BIT3_TRACE(("PIO allocated from: %x regs  %x count %x(%d) off %x\n"
		,uio_n_map,uio_n_len,count,count,uio_off));
	return(0);
} /* end Va_pio_map() */

static int Va_pio_access(u32 pos,int count,char *buf,int rw)
{
	int i,start,off,map;
	short	sval;
	long	lval;
	char *fun;
	start = 0x1000 - uio_off;
	map	= uio_n_map;
	off	= uio_off;
	uio_rw = rw;
	if(( rw == VME_WRITE) && (vme_mod == 2)) {fun = "write_16";goto write_16;}
	if(( rw == VME_WRITE) && (vme_mod == 4)) {fun = "write_32";goto write_32;}
	if(( rw == VME_READ ) && (vme_mod == 4)) {fun = " read_32";goto read_32; }
	if(( rw == VME_READ ) && (vme_mod == 2)) {fun = " read_16";goto read_16; }
	return(-EINVAL);
write_16:
	BIT3_TRACE(("PIO %s\n",fun));
	for(i = 0;i < count;i = i + vme_mod)
	{
		if(bit3_error)
		{
			count = i;
			bit3_error = 0;
			goto clean_pio;
		}
		Vm_get_uval(sval,(short *)((int ) buf + i));
		writew(sval,bit3_base3 
			+ map * 0x1000
			+ off
			+ i);
		if(bit3_pio_delay) udelay(bit3_pio_delay);
	} /* end for w16... count */
	goto	clean_pio;
write_32:
	BIT3_TRACE(("PIO %s\n",fun));
	for(i = 0;i < count;i = i + vme_mod)
	{
		if(bit3_error)
		{
			count = i;
			bit3_error = 0;
			goto clean_pio;
		}
		Vm_get_uval(lval,(long *)((int ) buf + i));
		writel(lval,bit3_base3 
			+ map * 0x1000
			+ off
			+ i);
		if(bit3_pio_delay) udelay(bit3_pio_delay);
	} /* end for w16... count */
	goto	clean_pio;
read_16:
	BIT3_TRACE(("PIO %s\n",fun));
	for(i = 0;i < count;i = i + vme_mod)
	{
		if(bit3_error)
		{
			count = i;
			bit3_error = 0;
			goto clean_pio;
		}
		sval = readw(bit3_base3 
				+ map * 0x1000
				+ off
				+ i);
		Vm_put_uval(sval, (unsigned short *)((int)buf + i));
		if(bit3_pio_delay) udelay(bit3_pio_delay);
	} /* end for r16... count */
	goto	clean_pio;
read_32:
	BIT3_TRACE(("PIO %s\n",fun));
	for(i = 0;i < count;i = i + vme_mod)
	{
		if(bit3_error)
		{
			count = i;
			bit3_error = 0;
			goto clean_pio;
		}
		lval = readl(bit3_base3 
				+ map * 0x1000
				+ off
				+ i),
		Vm_put_uval( lval, (unsigned long *)((int)buf + i));
		if(bit3_pio_delay) udelay(bit3_pio_delay);
	} /* end for r32... count */
	goto	clean_pio;
clean_pio:
	/* Bit3_map_free (bit3_base2,MR_PCI_VME,uio_n_len); /* DEBUG */
	return(count);
}/* end of Va_pio_access(u32 pos,int count,char *buf,int rw) 	*/

static int Va_map_map(u32 pos,int count)
{
	int	i;
	int cnt,val,err;
	cnt		= count;
	uio_off		= pos & 0xfff;
	uio_n_len	= ((pos + count ) >> 12) -
                          ( pos           >> 12)  ;
	if(count &0xfff)
		uio_n_len++;
	if(bit3_dma_busy)
	{
		/* we come here only to implement readahead  NIY	*/
		/* down(&bit3_sem)	.... e.t.c, e.t.c.... 		*/
	}
	uio_n_map	= Bit3_map_find(bit3_base2,MR_PCI_VME,uio_n_len,1);
	val = BIT3REG((u32)pos,vme_am,vme_fu,vme_swap,0); /**/
	for(i = 0;i < uio_n_len;i++)
	{
		err=Bit3_p_set(bit3_base2,MR_PCI_VME,uio_n_map + i,val);
		val += 0x1000;
	}
	VMEHB_TRACE(("MAP allocated from: %x regs  %x count %x(%d) off %x\n"
		,uio_n_map,uio_n_len,count,count,uio_off));
	return(uio_n_map);
} /* end Va_map_map() */
#ifdef NOT_YET_WRITTEN_ROUTINES
#endif
