#ifndef	__VMEHB_CONF_H
#define	__VMEHB_CONF_H
/*
    vmehb - VME bus driver of PCI/VME bus adaptor Model 617 of Bit3.
    ----------------------------------------------------------------
    driver configuration file 
    The file covers the following aspects:
    	Hardware dependency:
    		current configuration of VME
    	System dependency:
    		including proper system files
   		system independent wrappers for routines & macros
    	System tracing and debugging:
    This file contains VARIABLES !!!!

    Copyright (C) May 1997  Natalia Kruszynska <natalia@nikhef.nl>
	Thanks for contribution to:
		Andries Brouwer <aeb@cwi.nl>
		Marcin Dalecki  <dalecki@sub944.sub.uni-goettingen.de>
		David Grothe	<dave@gcom.com>
		Henk Peek	<henkp@nikhef.nl>
		Ruud van Wijk   <ruud@nikhef.nl>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 * The following  prefixes are consequently used in the code:
 *	vme_		- configuration data of a vme space
 *	bit3_		- device  structure members
 *	uio_		- request structure members (UNIX hickup)
 *	vmehb		- big routines of the driver 
 *      Va_		- base auxiliary routines
 *	Vm_		- semaphoring & setup inlines  
 *	Bit3_		- hardware access inline primitives
 *	set_		- different levels of tracing, defined below
 */

#	include	<linux/module.h>
#	include	<linux/wrapper.h>
#include	<linux/sched.h>		/* for resources routine */
#include	<linux/kernel.h>
#include	<linux/malloc.h>
#include	<linux/types.h>
#include	<linux/ioport.h>	/* for port operations	*/
#include	<linux/fs.h>
#include	<linux/mm.h>
#include	<linux/errno.h>
#include	<asm/io.h>		/* for port operations	*/
#include	<linux/bios32.h>	/* for pci_bios & probe */
#include	<linux/pci.h>
#include	<asm/segment.h>  	/* for get/put into user space	*/
#include	<asm/irq.h>		 /* for interrupt	*/
#include	<linux/delay.h>		 /* for udelay(usecs)	*/
#include	<linux/wait.h>		 /* for semaphoring	*/
#include	<linux/smp.h>		 /* for semaphoring	*/

/* SYSTEM COMPATIBILITY */
/*
 *  System compatibility, we will see wheter Vm..from.to  are the same
 *  in both Linux lines
 *      Vm_copy..io	- for read/write
 *      Vm_copy..user   - for ioctl
 *  let's assume they are the same...
 */
#ifdef LIN2_0 /* in 2_0_0 and 2_0_30 is the same */
#define	Vm_copy_to_user		memcpy_tofs
#define	Vm_copy_from_user	memcpy_fromfs
#define	Vm_copy_fromio		memcpy_tofs
#define	Vm_copy_toio		memcpy_fromfs
#define	Vm_get_uval(a,b)  	(a) = get_user((b))
#define	Vm_put_uval(a,b)  	put_user((a),(b))
#endif
#ifdef LIN2_1_27
/* changes from 2.0.0 to 2.1.27					*/
#include	<asm/uaccess.h>		/* VERIFY_READ/WRITE flags are here */
#define	vremap			ioremap
#define vfree			iounmap
/* see whether it works */
#define	Vm_copy_to_user		memcpy_fromio
#define	Vm_copy_from_user	memcpy_toio
#define	Vm_copy_fromio		memcpy_fromio
#define	Vm_copy_toio		memcpy_toio
#define	Vm_get_uval(a,b) 	get_user((a),(b))
#define	Vm_put_uval(a,b) 	put_user((a),(b))
#endif
/*
 * Compile time patching for ioctl & debugging flags
 */
/* fine VMEHB_DEBUG			/* debugging routines	*/
#define BIT3_DEBUG			/* debugging primitives	*/
/* fine SYS_TRACELEN		0x80	/* tracing length 	*/

/* do not change this : 					*/
#ifdef	VMEHB_DEBUG
#	define	VMEHB_TRACE(x)	printk x
#else
#	define	VMEHB_TRACE(x)	
#endif
#ifdef	BIT3_DEBUG
#	define	BIT3_TRACE(x)	printk x
#else
#	define	BIT3_TRACE(x)	
#endif
#ifdef	SYS_DEBUG
#	define	SYS_TRACE(x)	printk x
#else
#	define	SYS_TRACE(x)	
#endif

#define	NO_SA_INTERRUPT	0

/* local include files */
/* ------------------------------------------------------------- */
#include	"bit3_def.h"		/* soft defs & primitives */
#include	"bit3_reg.h"		/* hard defs & primitives */
#include	"vmehb_ioctl.h"		/* ioctl defs 		  */

/* saving state of the device here */
static  pci_config_t	PCI_conf;
static	int 		bit3_major	= BIT3_MAJOR;
static	int 		bit3_irq	= BIT3_IRQ;

/* ==================================================================
** bit3 device parameters access 			
** here we can extend for multi-device access	
---------------------------------------------------------------------	*/
static struct bit3_dev	bit3_dev;
struct	bit3_dev 	*bit3_dev_p	= (struct bit3_dev *)&bit3_dev;
struct	io_ahead 	*uio_p; 

/* ------------------------------------------------------------------ */
/* VME parameters access */
int	am_defaults[10] = {0x2d,0x2d,0x3d,0x3d,0xd,0xd,0,0,0,0};
int	sw_defaults[10] = {0,0,0,0,0,0,0,0,0,0};
int	vm_defaults[10] = {0,0,0x40,0x40,0x40,0x40,0,0,0,0};
#define	vme_minor	MINOR(node->i_rdev)
/*
** VME config   - max PIO can not exceed 0x1000 ( one VME page size )
**	name 	- standard name  ( should keep within 10 chars )
**	minor	- minor number of the device
**	am	- address modifier
**	mod	- length of data step in bytes
**	dmamod	- BLOCK PAUSE or NON_BLOCK
**	swap	- 3 bits constant defining swap - 0= no swap at all
**	fu	- 1 - VME I/O ,3 Dual Port I/O, 2 VME RAM I/O
**	pio	- up to this length DMA is not bothered
**	maxad	- max addr of a given space (VME ADDR or Dual Port length)
*/
struct vme_dev	vme_config[] = {
/* name    minor   am    mod    dmamod 		swap	fu      pio ,max	*/ 
{"vme16d16",   0, 0x2d,	 2,	RC2_DMA_PAUSE    ,0,	1,	256 ,0x0000ffff},
{"vme16d32",   1, 0x2d,	 4,	RC2_DMA_PAUSE    ,0,	1,	256 ,0x0000ffff},
{"vme24d16",   2, 0x3d,	 2,	RC2_DMA_PAUSE    ,0,	1,	256 ,0x00ffffff},
{"vme24d32",   3, 0x3d,	 4,	RC2_DMA_PAUSE    ,0,	1,	256 ,0x00ffffff},
{"vme32d16",   4, 0x0d,	 2,	RC2_DMA_PAUSE    ,0,	1,	256 ,0xffffffff},
{"vme32d32",   5, 0x0d,	 4,	RC2_DMA_PAUSE    ,0,	1,	256 ,0xffffffff},
/* dual port access */
{"vmedpd16",   6, 0x0d,	 2,	0,	       	  0,	3,	256 ,0x07ffffff},
{"vmedpd32",   7, 0x0d,	 4,	0,		  0,	3,	256 ,0x07ffffff},
/* vme ram   access */
{"vmerad16",   8, 0x0d,	 2,	0,		  0,	2,	256 ,0x07ffffff},
{"vmerad32",   9, 0x0d,	 4,	0,		  0,	2,	256 ,0x07ffffff},
/* vme block access */
{"vmf24d16",  10, 0x3f,	 2,	RC2_DMA_BLK_SEL,  0,	1,	256 ,0x00ffffff},
{"vmf24d32",  11, 0x3f,	 4,	RC2_DMA_BLK_SEL,  0,	1,	256 ,0x00ffffff},
{"vmf32d16",  12, 0x0f,	 2,	RC2_DMA_BLK_SEL,  0,	1,	256 ,0xffffffff},
{"vmf32d32",  13, 0x0b,	 4,	RC2_DMA_BLK_SEL,  0,	1,	256 ,0xffffffff},
/* exotic VME access single user devices	*/
{"vmfunny1",  14, 0x3d,	 4,	RC2_DMA_PAUSE    ,0,	1,	256 ,0xffffffff},
{"vmfunny2",  15, 0x3d,	 4,	RC2_DMA_PAUSE    ,0,	1,	256 ,0xffffffff},
};
struct	vme_dev	*vme_dev_p; /* = struct vme_dev *)&vme_config[vme_minor])  /**/

/* ================================================================== */
/* Dma Memory related stuff from floppy driver */

/* Pure 2^n version of get_order */
static inline int __get_order(unsigned long size)
{
	int order;

	size = (size-1) >> (PAGE_SHIFT-1);
	order = -1;
	do {
		size >>= 1;
		order++;
	} while (size);
	return order;
}
static inline unsigned long Vm_dma_mem_alloc(unsigned long size)
{
	return virt_to_bus((volatile void *)__get_dma_pages(GFP_KERNEL,__get_order(size)));
}

static inline void Vm_dma_mem_free(unsigned long addr, unsigned long size)
{
	free_pages((unsigned long)bus_to_virt(addr), __get_order(size));
}
/* 
 * Semaphores
 */

static  inline void Vm_down(struct semaphore *sem)
{
	down(sem);
} /* end Vm_down() */
static inline void Vm_up(struct semaphore *sem)
{
	up(sem);
} /* end Vm_up() */
/* 
 *   bit3_sem      semaphore for the initialization by current MUTEX
 *   bit3_wakeups  	clear flag
 *   bit3_error    		clear flag
 *   bit3_slow_control  	1 to avoid DMA, 0 for DATA mode
 *   bit3_irq_used 		flag to switch the mode of DMA service
 *	1		DMA interrupt mode
 *      0               DMA poll busy a lot....
 *	-1000-0		DMA poll with udelay of -flag between polls
 *      -1000 >		Fancy modes for tests, to try interrupts
 *   bit3_poll_delay	Check VME validity mode (PIO only)
 */
static inline void Vm_report_driver()
{
	/* here we place info over which testing version it is */
	if(bit3_pio_delay)
		printk("Check VME mode : PIO only , delayed %d usecs\n"
			,bit3_pio_delay);
	if(bit3_slow_control)
		printk("Slow control mode: PIO only, mmap allowed\n");
	/* two models allowed for DMA access */
	if(bit3_irq_used > 0)
		printk("IRQ - 2.0 DMA serialized with semaphores  \n"); /* IRQ */
	if(bit3_irq_used <= 0 )
		printk("Test version: POLL - DMA serialized with semaphores \n"); /*POLL*/
	if(bit3_VME_on == 0)
	{
		printk("VME crate off!!, every access will return ENODEV\n");
	}
	return;
}
static inline void Vm_set_me_up()
{
	/* Vm_set_me_up				*/
	/* preset PIO registers with 1		*/
	Bit3_map_free(bit3_base2,MR_PCI_VME,(MR_PCI_VME_SIZE) >> 2);
	/* initialize the device structure	*/
	bit3_sem 		= MUTEX;
	bit3_wakeups 		= 0;
	bit3_error		= 0;
					/* mmap'ers adm		*/
	bit3_first		= (struct vme_frag *)0;
	bit3_next		= bit3_first;
					/* driver options	*/
	bit3_slow_control	= 1;	/* SLOW CONTROL */
	bit3_slow_control	= 0;	/* DMA ALLOWES	*/

	bit3_irq_used		= -2000;/* POLL with schedule() after jiffie */
        bit3_irq_used		= -1;	/* POLL */
        bit3_irq_used		=  0;	/* POLL  deadly busy*/
        bit3_irq_used		= 1;	/* DMA  */

	bit3_pio_delay		= 50;	/* VME BERR CHECK MODE	*/
	bit3_pio_delay 		= 0;	/* VME BERR not fixed   */

	Vm_report_driver();
} /* end Vm_set_me_up() */
static inline void Vm_list_maps()
{
	struct vme_frag *list;
	list	= bit3_first;
	while(list != (struct vme_frag *)0 )
	{
		printk("vmehb: %8x %4d   maps %8x from %4d len %d\n"
			,(int)list
			,list->pid
			,list->vadr
			,list->from
			,list->pages
			);
		list	= list->next;
	}
	Bit3_map_dump(bit3_base2,MR_PCI_VME + 8,16);
}
static inline void Vm_clear_maps(int pid)
{
	struct	vme_frag	*old;
	struct	vme_frag	*next;
	next	= bit3_first;
	old	= bit3_first;
	if(pid == 0)
	{
		/* free all maps	*/
		printk("vmehb: clears all maps\n");
		Bit3_map_free(bit3_base2,MR_PCI_VME,(MR_PCI_VME_SIZE) >> 2); /**/
	}
	while(next != 0)
	{
		/* pid 0 clears all maps			 */
		if((pid) && next->pid != pid)
		{
			next = next->next;
			continue;
		}
		if(pid) 
		{
			/* free this map	*/
			printk("vmehb: %8x %4d unmaps %8x from %4d len %d\n"
				,(int)next
				,next->pid
				,next->vadr
				,next->from
				,next->pages
				);
			Bit3_map_free(bit3_base2
				,MR_PCI_VME + ((next->from) << 2)
				,next->pages
				); /**/
		}
		if(next == bit3_first)
		{
			/* the first	*/
			BIT3_TRACE(("At the head from %d \n",next->from));
			kfree (bit3_first);	/*	*/
			bit3_first	= next->next;
			next		= next->next;
		}
		else
		{
			/* another	*/
			BIT3_TRACE(("Another     from %d\n",next->from));
			old->next 	= next->next;
			kfree( next);		/*	*/
			next 		= next->next;
			old 		= next->next;
		}
	}
}

/*
 * SYSTEM TRACING: reported when the power goes off VME crate
 * and by the ioctl VME_TRACE_INFO ( no params , any device )
 * Here you can see how to add your own level. Try it once...
 *   Till now the two sets of functions were used, the first
 *   with three flags, another only with values
 *	set_A	- tracing of driver occasional coma problem
 *	set_B   - tracing of get_dma_pages *0, which 
 *                 appeared once
 */
#ifdef SYS_TRACELEN
static	int 	trace[SYS_TRACELEN];
static	int	traceno 	= 0;
/* TRACING A - of critical leaks  causing coma of driver 
   let me find the critical section not secured by cli() */
/* TRACING B - of dma buffer memory leak	*/
/* TRACING C - not used yet              */
static	int	dma_r_cnt	= 0;
static	int	dma_w_cnt	= 0;
static	int	irq_d_cnt	= 0;
#define	more_rdma	++dma_r_cnt
#define	more_wdma	++dma_w_cnt
#define	more_irq	++irq_d_cnt
#define	set_B(val)	
#define	set_A(val)	
static  inline  set_C(val)	{
	trace[traceno++] = (unsigned int)val;
	if(traceno > SYS_TRACELEN - 1)
		traceno = 0;
};
static void print_trace()
{
	int i;
	printk("Current trace : %x\n",traceno);
	for(i = 0;i < SYS_TRACELEN;i+= 8)
	{
		printk("%3x: %8x %8x %8x %8x|%8x %8x %8x %8x\n"
			,i
			,trace[i]
			,trace[i + 1]
			,trace[i + 2]
			,trace[i + 3]
			,trace[i + 4]
			,trace[i + 5]
			,trace[i + 6]
			,trace[i + 7]
			);
	}
}
#else
#define	set_A(val)	
#define	set_B(val)	
#define	set_C(val)	
#define	print_trace()
#define	more_rdma	
#define	more_wdma
#define	more_irq
#endif

#endif 	/* __VMEHB_CONF_H /**/
