#ifndef VMEHB_IOCTL_H
#define VMEHB_IOCTL_H
/* 
    vmehb - VME bus driver of PCI/VME bus adaptor Model 617 of Bit3.
    ----------------------------------------------------------------
    This file contains the ioctl structures and should be made available
    to the users of the driver ( e.g. /usr/include/local ). It is used
    by the driver itself as well.
    Copyright (C) May 1997  Natalia Kruszynska <natalia@nikhef.nl>
	Thanks for contribution to:
		Andries Brouwer <aeb@cwi.nl>
		Marcin Dalecki  <dalecki@sub944.sub.uni-goettingen.de>
		David Grothe	<dave@gcom.com>
		Henk Peek	<henkp@nikhef.nl>
		Ruud van Wijk   <ruud@nikhef.nl>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef __KERNEL__ 
/****************************************************************
** Configuration info format of vme minors +  flags definitions
----------------------------------------------------------------*/
struct vme_dev {
/* VME devices configuration data */
	char 	name[10];
	int	minor;
	int	am;	/* VME address modifier			*/
	int	mod;	/* 2 or 4				*/
	int	dmamod;	/* DMA MODE flag value,see below	*/
	int	swap;   /* SWAP flags, see below		*/
	int	fu;	/* 1 - VME 2 - Dual Port 3 - VME RAM	*/
	int	pio;	/* MAX len serviced by PIO < 0x1000	*/
	unsigned long	
		maxad;	/* MAX addressing,eg ffff for a16	*/
};
/* SWAP flags , can be |' ed 			*/
#define	NO_SWAP			0
#define	BYTE_SWAP_NON_BYTE	1
#define	WORD_SWAP		2
#define	BYTE_SWAP_BYTE_D	4

/* DMA MODE flag value				*/
#define	NO_BLOCK		0
#define	RC2_DMA_BLK_SEL		(1 << 5)
#define	RC2_DMA_PAUSE  		(1 << 7)
/****************************************************************/
#endif

/* registered command & function range 		*/
#define VMEIOC		'b'
#define VMEMINFUN 	0x0
#define VMEMAXFUN 	0x3f

/* setting driver mode parameters   */
#define	VME_SET_MODE		1
#define	VME_CLR_MODE		0
#define VME_CHK_MODE		50 /* standard PIO delay in usecs */

/* 
 *  ioctls 0 and 1 allow to change the choosen parameters
 *  of the setting of vme spaces. The partial action is
 *  put on by ioctl's 5-6. Set actions are for su only.
 *  who takes the responsibility of making it o.k. 
 *  The best way is to get the structure and change as
 *  little as possible and to put permanent changes in the
 *  compile time in vmehb_conf.h
 */

#define VME_GET_SPACE		_IOR(VMEIOC,0,struct vme_dev)
#define VME_SET_SPACE		_IOW(VMEIOC,1,struct vme_dev)

/*
 *  ioctl 2 with VME_CHK_MODE puts the driver into diagnostic
 *  mode, i/e all the devices are checked under memory mapped
 *  access, delayed so that the VME BERR is diagnosed o.k.
 *          with VME_CLR_MODE puts the choice further
 */
#define	VME_CHECK_ACCESS	_IOW(VMEIOC,2,int) 

/*
 * ioctl 3 with VME_SET_MODE puts the driver into slow 
 * control mode, PIO only but no delay, mmap allowed
 *         with VME_CLR_MODE the su chooses the standard DACQ
 *	   mode with DMA, but no mmap.
 */
#define VME_SLOW_CONTROL	_IOW(VMEIOC,3,int)

/*
 * ioctl 4 with VME_SET MODE chooses IRQ as DMA service ( default)
 *         with VME_CLR_MODE chooses polling for DMA done
 */
#define	VME_DMA_CONTROL		_IOW(VMEIOC,4,int)


/*
 *  ioctl 5 takes as the parameter VME ADDRESS MODIFIER 
 * ( see specs of VME bus)
 */
#define	VME_SET_AM		_IOW(VMEIOC,5,int)

/*
 *   DMA on VME side can work in 3 modes, take care that the mode
 *   fits the ADDRESS MODIFIER of the VME device accessed
 *   	block 	  - arbitrage each 256 bytes RC2_DMA_BLK_SEL
 *	pause 	  - arbitrage each  64 bytes RC2_DMA_PAUSE
 *      non block - arbitrage each transfer  VME_CLR_MODE
 */
#define VME_SET_DMAMOD		_IOW(VMEIOC,6,int)

/*
 *	ioctl 10 and 11 are returning success on the current
 *	state of power on VME crate, error if it does not fit.
 *	su changes to the requested state.
 */
#define VME_CRATE_OFF		_IO(VMEIOC,10)
#define	VME_CRATE_ON		_IO(VMEIOC,11)

/*
 * 	ioctl 61 frees all the pio memory 
 */
#define	VME_CLEAR_MAPS		_IO(VMEIOC,61)

/*
 *	ioctl 62 ( the one before the last reserved) is used to 
 *	trace ev deadlocks of driver( so called "coma").  
 */
#define	VME_TRACE_ZOMBIES	_IO(VMEIOC,62)

/*
 *	ioctl 63 ( the last reserved ) is triggering the system 
 *      trace (if any)	into /var/log/syslog
 */
#define	VME_TRACE_INFO		_IO(VMEIOC,63)

/*     VMEHB_IOCTL_H */
#endif
